#!/usr/bin/env python3.5
# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2023 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Redis Queue server custom worker
"""
from rq import Connection, Worker
from redis import Redis

# We need to append the path to the actinia_core package, since
# flask API does not send the correct module and package paths
# to the worker, so the workers are unable to de-serialize
# that object that are required by the asynchronous process classes.

# import sys
# import actinia_core
# from os.path import dirname
# sys.path.append(dirname(actinia_core.__file__))
# Integrate the fluentd logger into the logging infrastructure
# https://github.com/fluent/fluent-logger-python
import logging
import logging.handlers
from actinia_core.core.common.config import Configuration
from actinia_core.core.logging_interface import log
import os
import argparse
import platform

has_fluent = False

try:
    from fluent import handler

    has_fluent = True
except Exception:
    print("Fluent is not available")
    has_fluent = False

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-2023, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


def main():
    parser = argparse.ArgumentParser(
        description="Start a single Actinia Core "
        "custom worker listening to a specific queue."
        "It uses the logfile settings that are specified "
        "in the default Actinia Core configuration file"
        "or a file specified by an optional path."
    )

    if parser.prog == "rq_custom_worker":
        log.warning(
            'The command "rq_custom_worker" is deprecated and will be '
            'removed soon. Use "actinia-worker" instead!'
        )

    parser.add_argument(
        "queue",
        type=str,
        help="The name of the queue that should be listen to by the worker",
    )
    parser.add_argument(
        "-c",
        "--config",
        type=str,
        required=False,
        help="The path to the Actinia Core configuration file",
    )
    parser.add_argument(
        "-q",
        "--quit",
        action="store_true",
        required=False,
        help="Wether or not the worker should exit when the queue is emptied.",
    )

    args = parser.parse_args()

    conf = Configuration()
    try:
        if args.config and os.path.isfile(args.config):
            conf.read(path=args.config)
        else:
            conf.read()
    except IOError as e:
        print(
            "WARNING: unable to read config file, "
            "will use defaults instead, IOError: %s" % str(e)
        )

    # Provide queue names to listen to as arguments to this script,
    # similar to rq worker
    with Connection(
        Redis(
            conf.REDIS_QUEUE_SERVER_URL,
            conf.REDIS_QUEUE_SERVER_PORT,
            password=conf.REDIS_QUEUE_SERVER_PASSWORD,
        )
    ):
        logger = logging.getLogger("rq.worker")
        logger.setLevel(logging.ERROR)

        node = platform.node()

        if conf.LOG_INTERFACE == "fluentd" and has_fluent is True:
            custom_format = {
                "host": "%(hostname)s",
                "where": "%(module)s.%(funcName)s",
                "status": "%(levelname)s",
                "stack_trace": "%(exc_text)s",
            }
            fh = handler.FluentHandler(
                "%s::rq.worker" % node,
                host=conf.LOG_FLUENT_HOST,
                port=conf.LOG_FLUENT_PORT,
            )
            formatter = handler.FluentRecordFormatter(custom_format)
            fh.setFormatter(formatter)
            logger.addHandler(fh)

        # Add the log message handler to the logger
        log_file_name = "%s_%s.log" % (conf.WORKER_LOGFILE, args.queue)
        lh = logging.handlers.RotatingFileHandler(
            log_file_name,
            maxBytes=2000000,
            backupCount=5,
        )
        logger.addHandler(lh)
        logger.fatal(
            msg="Started rq.worker: %s\n"
            "host %s port: %s \n"
            "logging into %s"
            % (
                args.queue,
                conf.REDIS_QUEUE_SERVER_URL,
                conf.REDIS_QUEUE_SERVER_PORT,
                log_file_name,
            )
        )

        actinia_worker = Worker(
            [
                args.queue,
            ]
        )
        if bool(args.quit) is True:
            actinia_worker.work(burst=True)
        else:
            actinia_worker.work()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: process queue test queue
"""
import unittest
import time
import datetime
from copy import deepcopy
from actinia_core.core.common.process_queue import (
    create_process_queue,
    enqueue_job,
    stop_process_queue,
)
from actinia_core.core.resource_data_container import ResourceDataContainer
from actinia_core.core.common.app import flask_app

try:
    from .test_resource_base import global_config
except ModuleNotFoundError:
    from test_resource_base import global_config

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


def job_with_exception(rdc):
    print("job_with_exception", rdc.api_info, rdc.orig_time)
    # time.sleep(3)
    # raise Exception("job_with_exception")


def job_short_run(rdc):
    for i in range(3):
        print("job_short_run", rdc.api_info, rdc.orig_time)
        # time.sleep(1)


def job_long_run(rdc):
    for i in range(8):
        print("job_long_run", rdc.api_info, rdc.orig_time)
        time.sleep(3)


class ProcessQueueTestCase(unittest.TestCase):
    """
    This class tests the api logging interface
    """

    def setUp(self):
        # We need to set the application context
        self.app_context = flask_app.app_context()
        self.app_context.push()
        # The test user
        self.user_id = "soeren"

        global global_config
        global_config.NUMBER_OF_WORKERS = 1

        self.rdc = ResourceDataContainer(
            grass_data_base="grass_data_base",
            grass_user_data_base="grass_user_data_base",
            grass_base_dir="grass_base_dir",
            request_data={"request_data": None},
            user_id="user_id",
            user_group="user_group",
            user_credentials={"user_credentials": None},
            resource_id="resource_id",
            status_url="status_url",
            api_info="api_info",
            resource_url_base="resource_url_base",
            orig_time=time.time(),
            orig_datetime=datetime.datetime.now(),
            config=global_config,
            location_name="location_name",
            mapset_name="mapset_name",
            map_name="map_name",
        )

    def tearDown(self):
        self.app_context.pop()

    def otest_1(self):
        create_process_queue(config=global_config, use_logger=False)

        args = deepcopy(self.rdc)
        args.api_info = 0

        args = deepcopy(self.rdc)
        args.api_info = 1
        enqueue_job(15, job_with_exception, args)
        args = deepcopy(self.rdc)
        args.api_info = 2
        enqueue_job(15, job_with_exception, args)
        args = deepcopy(self.rdc)
        args.api_info = 3
        enqueue_job(1, job_with_exception, args)
        args = deepcopy(self.rdc)
        args.api_info = 4
        enqueue_job(1, job_with_exception, args)
        args = deepcopy(self.rdc)
        args.api_info = 5
        enqueue_job(1, job_with_exception, args)
        args = deepcopy(self.rdc)
        args.api_info = 6
        enqueue_job(15, job_with_exception, args)
        args = deepcopy(self.rdc)
        args.api_info = 7
        enqueue_job(15, job_with_exception, args)
        args = deepcopy(self.rdc)
        args.api_info = 8
        enqueue_job(15, job_with_exception, args)

        time.sleep(10)
        stop_process_queue()

        return

    def otest_2(self):
        create_process_queue(config=global_config, use_logger=False)

        args = deepcopy(self.rdc)
        args.api_info = 0

        enqueue_job(30, job_with_exception, args)

        args = deepcopy(self.rdc)
        args.api_info = 1

        enqueue_job(30, job_short_run, args)

        args = deepcopy(self.rdc)
        args.api_info = 2

        enqueue_job(30, job_short_run, args)

        args = deepcopy(self.rdc)
        args.api_info = 3

        enqueue_job(30, job_short_run, args)

        args = deepcopy(self.rdc)
        args.api_info = 4

        enqueue_job(1, job_short_run, args)

        args = deepcopy(self.rdc)
        args.api_info = 5

        enqueue_job(30, job_long_run, args)

        args = deepcopy(self.rdc)
        args.api_info = 6

        enqueue_job(30, job_long_run, args)

        time.sleep(20)
        stop_process_queue()
        # return


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: List raster layers test case
"""
from flask.json import loads as json_loads, dumps as json_dumps
import unittest

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"


class ListRasterLayersTestCase(ActiniaResourceTestCaseBase):
    def create_raster_layer(self, mapset_name, raster_name):
        # Remove potentially existing raster layer
        rv = self.server.delete(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/%s"
            % (mapset_name, raster_name),
            headers=self.user_auth_header,
        )
        # print(rv.data)
        # Create
        postbody = {
            "list": [
                {
                    "id": "set_region",
                    "module": "g.region",
                    "inputs": [
                        {"param": "n", "value": "228500"},
                        {"param": "s", "value": "215000"},
                        {"param": "e", "value": "645000"},
                        {"param": "w", "value": "630000"},
                        {"param": "ewres", "value": "50"},
                        {"param": "nsres", "value": "50"},
                    ],
                },
                {
                    "id": "create_raster",
                    "module": "r.mapcalc",
                    "inputs": [
                        {
                            "param": "expression",
                            "value": "%s = 1" % raster_name,
                        }
                    ],
                },
            ],
            "version": "1",
        }
        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/processing_async" % mapset_name,
            headers=self.user_auth_header,
            data=json_dumps(postbody),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        # print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_list_raster_layers(self):
        rv = self.server.get(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/PERMANENT/raster_layers",
            headers=self.user_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        map_list = json_loads(rv.data)["process_results"]
        self.assertTrue("elevation" in map_list)
        self.assertTrue("aspect" in map_list)
        self.assertTrue("lsat7_2002_10" in map_list)
        self.assertTrue("slope" in map_list)

    def test_list_raster_layers_pattern(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/"
            "raster_layers?pattern=lsat*",
            headers=self.user_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        map_list = json_loads(rv.data)["process_results"]
        self.assertFalse("elevation" in map_list)
        self.assertFalse("aspect" in map_list)
        self.assertTrue("lsat7_2002_10" in map_list)
        self.assertFalse("slope" in map_list)

    def test_list_raster_layers_empty_list(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/"
            "raster_layers?pattern=NONE",
            headers=self.user_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        map_list = json_loads(rv.data)["process_results"]
        self.assertTrue(len(map_list) == 0)

    def test_remove_raster_layers_pattern(self):
        new_mapset = "raster_test_mapset"
        self.create_new_mapset(new_mapset)

        map_list = ["test_delete_layer_1", "test_delete_layer_2"]

        # Create raster layers
        for map_name in map_list:
            self.create_raster_layer(new_mapset, map_name)

        # # Delete raster layers
        # rv = self.server.delete(
        #     f"{URL_PREFIX}/locations/nc_spm_08/mapsets/user1/raster_layers?"
        #     "pattern=test_delete_layer_*",
        #     headers=self.user_auth_header
        # )
        # print(rv.data)
        # self.assertEqual(
        #     rv.status_code,
        #     200, "HTML status code is wrong %i" % rv.status_code
        # )
        # self.assertEqual(
        #     rv.mimetype,
        #     "application/json",
        #     "Wrong mimetype %s" % rv.mimetype
        # )

        # List raster layer
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/{new_mapset}/"
            "raster_layers?pattern=test_delete_layer_*",
            headers=self.user_auth_header,
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )
        # Check
        map_list_result = json_loads(rv.data)["process_results"]
        for map_name in map_list_result:
            self.assertTrue(map_name in map_list)

        # Delete raster layers
        for map_name in map_list:
            rv = self.server.delete(
                URL_PREFIX
                + "/locations/nc_spm_08/mapsets/%s/raster_layers/%s"
                % (new_mapset, map_name),
                headers=self.user_auth_header,
            )
            print(rv.data.decode())

    def test_rename_raster_layers(self):
        new_mapset = "raster_test_mapset"
        self.create_new_mapset(new_mapset)

        map_list = ["test_rename_layer_1", "test_rename_layer_2"]
        new_map_list = ["test_rename_layer_1_new", "test_rename_layer_2_new"]
        rename_map_list = [
            ("test_rename_layer_1", "test_rename_layer_1_new"),
            ("test_rename_layer_2", "test_rename_layer_2_new"),
        ]

        # Create raster layers
        for map_name in map_list:
            self.create_raster_layer(new_mapset, map_name)

        # Rename raster layer
        rv = self.server.put(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers" % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(rename_map_list),
            content_type="application/json",
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Rename raster layer
        rv = self.server.put(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers" % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(rename_map_list),
            content_type="application/json",
        )
        print(rv.data.decode())
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Delete raster layers
        for map_name in new_map_list:
            rv = self.server.delete(
                URL_PREFIX
                + "/locations/nc_spm_08/mapsets/%s/raster_layers/%s"
                % (new_mapset, map_name),
                headers=self.user_auth_header,
            )
            print(rv.data.decode())
            self.assertEqual(
                rv.status_code,
                200,
                "HTML status code is wrong %i" % rv.status_code,
            )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Base class for logger that uses fluentd/stdout for logging and resource storage
"""
import time
import logging

from actinia_core.core.common.config import global_config
from .logging_interface import log

if global_config.LOG_INTERFACE == "fluentd":
    from fluent import sender


__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-present, Sören Gebbert and mundialis GmbH & Co. KG"
)


class RedisFluentLoggerBase(object):
    """Base class for all logger that use Redis and fluentd for logging"""

    def __init__(self, config=None, user_id=None, fluent_sender=None):
        if config is None:
            config = global_config
        if user_id is None:
            user_id = "Unknown"

        self.host = config.LOG_FLUENT_HOST
        self.port = config.LOG_FLUENT_PORT
        self.interface = config.LOG_INTERFACE
        self.log_level = config.LOG_LEVEL
        self.user_id = user_id
        self.fluent_sender = fluent_sender

        # The fluentd sender
        if fluent_sender is None and global_config.LOG_INTERFACE == "fluentd":
            self.fluent_sender = sender.FluentSender(
                "actinia_logger", host=self.host, port=self.port
            )

    def _send_to_fluent(self, tag, data):
        try:
            cur_time = int(time.time())
            self.fluent_sender.emit_with_time(
                tag, timestamp=cur_time, data=data
            )
        except Exception as e:
            log.error(
                (
                    "%s is unable to connect to fluentd server host %s "
                    "port %i Error: %s, Content %s"
                )
                % (tag, self.host, self.port, str(e), str(data))
            )

        # keep this until sure that all logs are fetched if stdout log is set
        # tags = ['RESOURCE_LOG', 'API_LOG', 'INFO', 'DEBUG']
        if (
            "RESOURCE_LOG" not in tag
            and "API_LOG" not in tag
            and "INFO" not in tag
            and "DEBUG" not in tag
        ):
            print(
                "WARNING: Some output might not be redirected to STDOUT:"
                + " %s %s %s" % (tag, str(cur_time), str(data))
            )

    def _send_to_logging_interface(self, tag, data):
        if tag == "RESOURCE_LOG" and "status" in data:
            if data["status"] == "error":
                log.error(data)
            elif data["status"] == "terminated":
                log.warning(data)
            else:
                log.info(data)

        elif tag == "API_LOG":
            log.info(data)

        # MESSAGES_LOGGER
        else:
            try:
                log.log(getattr(logging, tag), data)
            except AttributeError:
                log.debug("Unknown log tag for logging: %s", tag)
                log.info(data)

    def send_to_logger(self, tag, data):
        if global_config.LOG_INTERFACE == "fluentd":
            self._send_to_fluent(tag, data)

        # always send to logger as file logger (and stdout) is included here
        self._send_to_logging_interface(tag, data)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Raster map renderer
"""
from flask import jsonify, make_response, Response
from actinia_core.core.common.redis_interface import enqueue_job
from actinia_core.rest.base.renderer_base import RendererBaseResource
import os
from flask_restful_swagger_2 import swagger
from actinia_api.swagger2.actinia_core.apidocs import strds_renderer

from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.processing.common.strds_renderer import start_job

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class SyncEphemeralSTRDSRendererResource(RendererBaseResource):
    @endpoint_decorator()
    @swagger.doc(check_endpoint("get", strds_renderer.get_doc))
    def get(self, location_name, mapset_name, strds_name):
        """
        Render the raster map layers of a specific STRDS as a single image.
        """
        parser = self.create_parser()
        args = parser.parse_args()
        options = self.create_parser_options(args)

        if isinstance(options, dict) is False:
            return options

        rdc = self.preprocess(
            has_json=False,
            has_xml=False,
            location_name=location_name,
            mapset_name=mapset_name,
            map_name=strds_name,
        )

        rdc.set_user_data(options)

        enqueue_job(
            self.job_timeout, start_job, rdc, queue_type_overwrite=True
        )

        http_code, response_model = self.wait_until_finish(0.05)
        if http_code == 200:
            result_file = response_model["process_results"]
            # Open the image file, read it and then delete it
            if result_file:
                if os.path.isfile(result_file):
                    image = open(result_file, "rb").read()
                    os.remove(result_file)
                    return Response(image, mimetype="image/png")
        return make_response(jsonify(response_model), http_code)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2021-present mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Upload vector via endpoint test case
"""
import os
import unittest

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except Exception:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX

__license__ = "GPLv3"
__author__ = "Anika Weinmann, Guido Riembauer"
__copyright__ = "Copyright 2016-2021, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

geojson_data = """{
  "type": "FeatureCollection",
  "features": [
    {
      "type": "Feature",
      "properties": {},
      "geometry": {
        "type": "Polygon",
        "coordinates": [
          [
            [
              -78.64837646484375,
              35.820040281161
            ],
            [
              -78.71429443359375,
              35.75765724051559
            ],
            [
              -78.63739013671874,
              35.71752800693265
            ],
            [
              -78.54400634765624,
              35.737595151747826
            ],
            [
              -78.55499267578125,
              35.808904044068626
            ],
            [
              -78.64837646484375,
              35.820040281161
            ]
          ]
        ]
      }
    }
  ]
}"""


class UploadVectorLayerTestCase(ActiniaResourceTestCaseBase):
    location = "nc_spm_08"
    mapset = "PERMANENT"
    tmp_mapset = "mapset_upload"
    vector = "testvector"
    local_geojson = f"/tmp/{vector}.geojson"
    gpkg_file = "/src/actinia_core/tests/data/nc_test_poly.gpkg"
    zipped_shp_file = "/src/actinia_core/tests/data/nc_test_poly.zip"

    ref_info = {
        "Attributes": [{"column": "cat", "type": "INTEGER"}],
        "areas": "1",
        "islands": "1",
        "boundaries": "1",
        "centroids": "1",
        "lines": "0",
        "map3d": "0",
        "nodes": "1",
        "points": "0",
        "primitives": "2",
        "attribute_layer_number": "1",
        "attribute_primary_key": "cat",
        "bottom": "0.000000",
        "top": "0.000000",
        "east": "650847.36413895",
        "north": "229689.570968999",
        "south": "218319.98552841",
        "west": "635437.976534827",
    }

    @classmethod
    def setUpClass(cls):
        # download a vector to re-upload it
        super(UploadVectorLayerTestCase, cls).setUpClass()

        # write GeoJSON
        with open(cls.local_geojson, "w") as geojson:
            geojson.write(geojson_data)

    @classmethod
    def tearDownClass(cls):
        # remove GeoJSON
        os.remove(cls.local_geojson)

    def setUp(self):
        # create new temp mapset
        super(UploadVectorLayerTestCase, self).setUp()
        self.create_new_mapset(self.tmp_mapset, location_name=self.location)

    def tearDown(self):
        # delete mapset
        self.delete_mapset(self.tmp_mapset, location_name=self.location)
        super(UploadVectorLayerTestCase, self).tearDown()

    def test_upload_vector_geojson_userdb(self):
        """
        Test successful GeoJSON upload and check against reference vector info
        """
        url = (
            f"{URL_PREFIX}/locations/{self.location}/mapsets/{self.tmp_mapset}"
            f"/vector_layers/{self.vector}"
        )
        multipart_form_data = {"file": open(self.local_geojson, "rb")}
        rv = self.server.post(
            url,
            content_type="multipart/form-data",
            headers=self.user_auth_header,
            data=multipart_form_data,
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=200,
            status="finished",
        )

        self.assertVectorInfo(
            self.location,
            self.tmp_mapset,
            self.vector,
            self.ref_info,
            self.user_auth_header,
        )

    def test_upload_vector_gpkg_userdb(self):
        """
        Test successful GPKG upload and check against reference vector info
        """
        url = (
            f"{URL_PREFIX}/locations/{self.location}/mapsets/{self.tmp_mapset}"
            f"/vector_layers/{self.vector}"
        )
        multipart_form_data = {"file": open(self.gpkg_file, "rb")}
        rv = self.server.post(
            url,
            content_type="multipart/form-data",
            headers=self.user_auth_header,
            data=multipart_form_data,
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=200,
            status="finished",
        )

        self.assertVectorInfo(
            self.location,
            self.tmp_mapset,
            self.vector,
            self.ref_info,
            self.user_auth_header,
        )

    def test_upload_vector_zipped_shp_userdb(self):
        """
        Test successful zipped Shapefile upload and check against reference
        vector info
        """
        url = (
            f"{URL_PREFIX}/locations/{self.location}/mapsets/{self.tmp_mapset}"
            f"/vector_layers/{self.vector}"
        )
        multipart_form_data = {"file": open(self.zipped_shp_file, "rb")}
        rv = self.server.post(
            url,
            content_type="multipart/form-data",
            headers=self.user_auth_header,
            data=multipart_form_data,
        )

        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=200,
            status="finished",
        )

        shp_ref_info = self.ref_info.copy()
        shp_ref_info["Attributes"] = [
            {"column": "cat", "type": "INTEGER"},
            {"column": "fid", "type": "DOUBLE PRECISION"},
        ]
        self.assertVectorInfo(
            self.location,
            self.tmp_mapset,
            self.vector,
            shp_ref_info,
            self.user_auth_header,
        )

    def test_upload_vector_globaldb_error(self):
        """Test Error if vector (GPKG) is uploaded to global DB"""
        url = (
            f"{URL_PREFIX}/locations/{self.location}/mapsets/{self.mapset}/"
            f"vector_layers/{self.vector}"
        )
        multipart_form_data = {"file": open(self.gpkg_file, "rb")}
        rv = self.server.post(
            url,
            content_type="multipart/form-data",
            headers=self.user_auth_header,
            data=multipart_form_data,
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=400,
            status="error",
            message_check=(
                f"Mapset <{self.mapset}> exists in the global "
                "dataset and can not be modified."
            ),
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Async process2 test case
"""
import configparser
import os
import unittest
from flask.json import dumps as json_dumps
from flask.json import loads as json_loads
from jinja2 import Template
from time import sleep

from actinia_core.core.common.config import global_config

try:
    from .test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
        additional_external_data,
    )
except ModuleNotFoundError:
    from test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
        additional_external_data,
    )

__license__ = "GPLv3"
__author__ = "Anika Weinmann"
__copyright__ = "Copyright 2021-2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"
__email__ = "info@mundialis.de"


process_chain_1 = {
    "version": 1,
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [{"param": "raster", "value": "elevation@PERMANENT"}],
            "flags": "p",
        },
        {
            "id": "2",
            "module": "r.mapcalc",
            "inputs": [{"param": "expression", "value": "baum = 5"}],
        },
        {
            "id": "3",
            "module": "r.info",
            "inputs": [{"param": "map", "value": "{{ map1 }}"}],
        },
        {
            "id": "4",
            "module": "g.list",
            "inputs": [{"param": "type", "value": "raster"}],
        },
        {
            "id": "5",
            "module": "r.info",
            "inputs": [{"param": "map", "value": "{{ map2 }}"}],
        },
    ],
}

process_chain_2_error = {
    "version": 1,
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [{"param": "raster", "value": "elevation@PERMANENT"}],
            "flags": "p",
        },
        {"id": "2", "exe": "sleep", "params": ["{{ seconds }}"]},
        {
            "id": "3",
            "module": "r.info",
            "inputs": [{"param": "map", "value": "elevation2@PERMANENT"}],
        },
    ],
}

process_chain_3_importer = {
    "version": 1,
    "list": [
        {
            "module": "g.region",
            "id": "1",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data["elev_ned_30m_tif"],
                        "type": "raster",
                    },
                    "param": "raster",
                    "value": "elev_ned_30m_new",
                }
            ],
            "flags": "p",
        },
        {
            "id": "2",
            "module": "r.mapcalc",
            "inputs": [{"param": "expression", "value": "baum = 5"}],
        },
        {
            "id": "3",
            "module": "r.info",
            "inputs": [{"param": "map", "value": "{{ map1 }}"}],
        },
        {
            "id": "4",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data["pointInBonn"],
                        "type": "vector",
                    },
                    "param": "map",
                    "value": "point",
                },
                {
                    "import_descr": {
                        "source": additional_external_data["geology_30m_tif"],
                        "type": "raster",
                    },
                    "param": "map",
                    "value": "geology_30m_new",
                },
            ],
        },
        {
            "id": "5",
            "module": "r.info",
            "inputs": [{"param": "map", "value": "{{ map2 }}"}],
        },
        {
            "id": "6",
            "module": "g.list",
            "inputs": [{"param": "type", "value": "raster"}],
        },
        {
            "id": "7",
            "module": "v.info",
            "inputs": [{"param": "map", "value": "point"}],
        },
        {
            "id": "8",
            "module": "r.info",
            "inputs": [{"param": "map", "value": "geology_30m_new"}],
        },
    ],
}


process_chain_4_exporter = {
    "version": 1,
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [{"param": "raster", "value": "elevation@PERMANENT"}],
            "flags": "p",
        },
        {
            "id": "2",
            "module": "r.colors.out_sld",
            "inputs": [{"param": "map", "value": "elevation@PERMANENT"}],
            "outputs": [
                {
                    "param": "output",
                    "value": "$file::export_of_2",
                    "export": {"type": "file", "format": "TXT"},
                }
            ],
        },
        {
            "id": "3",
            "module": "r.mapcalc",
            "inputs": [{"param": "expression", "value": "baum = 5*elevation"}],
        },
        {
            "id": "4",
            "module": "r.info",
            "inputs": [{"param": "map", "value": "{{ map1 }}"}],
        },
        {
            "id": "5",
            "module": "g.list",
            "inputs": [{"param": "type", "value": "raster"}],
        },
        {
            "id": "6",
            "module": "r.info",
            "inputs": [{"param": "map", "value": "{{ map2 }}"}],
        },
        {
            "id": "7",
            "module": "exporter",
            "comment": "Export indices as COG",
            "outputs": [
                {
                    "export": {"type": "raster", "format": "COG"},
                    "param": "map",
                    "value": "baum",
                },
                {
                    "export": {"type": "raster", "format": "COG"},
                    "param": "map",
                    "value": "elevation@PERMANENT",
                },
            ],
        },
    ],
}

process_chain_5_stdout = {
    "version": 1,
    "list": [
        {
            "id": "1",
            "module": "g.region",
            "inputs": [{"param": "raster", "value": "elevation@PERMANENT"}],
            "flags": "p",
        },
        {
            "id": "2",
            "module": "r.colors.out_sld",
            "inputs": [
                {"param": "map", "value": "elevation@PERMANENT"},
                {"param": "style_name", "value": "elevation"},
            ],
            "stdout": {"id": "sld", "format": "list", "delimiter": "\n"},
        },
        {
            "id": "3",
            "module": "r.mapcalc",
            "inputs": [{"param": "expression", "value": "baum = 5"}],
        },
        {
            "id": "4",
            "module": "r.info",
            "inputs": [{"param": "map", "value": "{{ map1 }}"}],
            "flags": "g",
            "stdout": {"id": "r_info_map1", "format": "kv", "delimiter": "="},
        },
        {
            "id": "5",
            "module": "g.list",
            "inputs": [{"param": "type", "value": "raster"}],
        },
        {
            "id": "6",
            "module": "r.info",
            "inputs": [{"param": "map", "value": "{{ map2 }}"}],
            "flags": "g",
            "stdout": {"id": "r_info_map2", "format": "kv", "delimiter": "="},
        },
    ],
}


class JobResumptionProcessingTestCase(ActiniaResourceTestCaseBase):
    cfg_file = os.environ.get(
        "ACTINIA_CUSTOM_TEST_CFG", "/etc/default/actinia"
    )
    tmp_cfg_file = "%s_tmp" % cfg_file
    save_interim_results_value = None
    endpoint = "/locations/nc_spm_08/processing_async"
    resource_user_id = None
    resource_resource_id = None
    sleep_time = 15
    save_interim_results = "True"

    @classmethod
    def save_config(cls, src, dest, value):
        config = configparser.ConfigParser()
        config.read(src)
        config["MISC"]["save_interim_results"] = value
        with open(dest, "w") as configfile:
            config.write(configfile)

    @classmethod
    def setUpClass(cls):
        # change save_interim_results in config to True
        cls.save_interim_results_value = global_config.SAVE_INTERIM_RESULTS
        if cls.save_interim_results_value is False:
            os.replace(cls.cfg_file, cls.tmp_cfg_file)
            cls.save_config(
                cls.tmp_cfg_file,
                cls.cfg_file,
                cls.save_interim_results,
            )

        super(JobResumptionProcessingTestCase, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        # reset config
        if cls.save_interim_results_value is False:
            cls.save_config(cls.tmp_cfg_file, cls.cfg_file, "False")
            os.remove(cls.tmp_cfg_file)
        global_config.read(cls.cfg_file)

        super(JobResumptionProcessingTestCase, cls).tearDownClass()

    def test_notsaved_interim_results_by_success(self):
        """Test if the interim results are not saved if process ends
        successfully
        """
        tpl = Template(json_dumps(process_chain_1))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

        # check if interim results are not saved
        resp_data = json_loads(rv.data)
        rv_user_id = resp_data["user_id"]
        rv_resource_id = resp_data["resource_id"]
        interim_dir = os.path.join(
            global_config.GRASS_RESOURCE_DIR,
            rv_user_id,
            "interim",
            rv_resource_id,
        )
        self.assertFalse(
            os.path.isdir(interim_dir),
            "Interim results are stored",
        )

    def test_saved_interim_results(self):
        """Test if the interim results are not saved correctly"""
        step = 4
        tpl = Template(json_dumps(process_chain_1))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum555"),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )

        # check if interim results are saved
        resp_data = json_loads(rv.data)
        rv_user_id = resp_data["user_id"]
        rv_resource_id = resp_data["resource_id"]
        interim_dir = os.path.join(
            global_config.GRASS_RESOURCE_DIR,
            rv_user_id,
            "interim",
            rv_resource_id,
        )
        self.assertTrue(
            os.path.isdir(interim_dir),
            "Interim results are not stored in the expected folder",
        )
        self.assertTrue(
            os.path.isdir(os.path.join(interim_dir, f"step{step}")),
            "Interim results mapset is not stored in the expected folder",
        )
        self.assertIn(
            "baum",
            os.listdir(os.path.join(interim_dir, f"step{step}", "cellhd")),
            "Raster map 'baum' not in interim results mapset",
        )
        self.assertTrue(
            os.path.isdir(os.path.join(interim_dir, f"tmpdir{step}")),
            "Interim results temporary file path is not stored in the expected"
            " folder",
        )

    def test_job_resumption(self):
        """Test job resumption with processing_async endpoint"""
        tpl = Template(json_dumps(process_chain_1))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation2@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )
        status_url = resp["urls"]["status"].split(URL_PREFIX)[-1]

        # check if interim results are saved
        resp_data = json_loads(rv.data)
        rv_user_id = resp_data["user_id"]
        rv_resource_id = resp_data["resource_id"]
        interim_dir = os.path.join(
            global_config.GRASS_RESOURCE_DIR,
            rv_user_id,
            "interim",
            rv_resource_id,
        )
        self.assertTrue(
            os.path.isdir(interim_dir),
            "Interim results are not stored in the expected folder",
        )

        # resumption of the job
        rv2 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv2,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    def test_job_2_times_resumption(self):
        """Test job 2 times resumption with processing_async endpoint"""
        tpl = Template(json_dumps(process_chain_1))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation2@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )
        status_url = resp["urls"]["status"].split(URL_PREFIX)[-1]

        # first job resumption
        rv2 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum1"),
            content_type="application/json",
        )
        resp2 = self.waitAsyncStatusAssertHTTP(
            rv2,
            headers=self.admin_auth_header,
            http_status=400,
            status="error",
        )
        status_url = resp2["urls"]["status"].split(URL_PREFIX)[-1]

        # check if interim results are saved
        resp_data = json_loads(rv2.data)
        rv_user_id = resp_data["user_id"]
        rv_resource_id = resp_data["resource_id"]
        interim_dir = os.path.join(
            global_config.GRASS_RESOURCE_DIR,
            rv_user_id,
            "interim",
            rv_resource_id,
        )
        self.assertTrue(
            os.path.isdir(interim_dir),
            "Interim results are not stored in the expected folder",
        )

        # second job resumption
        rv3 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv3,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        self.__class__.resource_user_id = rv_user_id
        self.__class__.resource_resource_id = rv_resource_id

    def test_job_resumption_error_by_running(self):
        """Test job resumption error by running process with processing_async
        endpoint
        """
        tpl = Template(json_dumps(process_chain_2_error))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(seconds=self.sleep_time),
            content_type="application/json",
        )
        resp_data = json_loads(rv.data)
        rv_user_id = resp_data["user_id"]
        rv_resource_id = resp_data["resource_id"]
        waiting_for_running = True
        while waiting_for_running:
            rv_pull = self.server.get(
                URL_PREFIX + "/resources/%s/%s" % (rv_user_id, rv_resource_id),
                headers=self.admin_auth_header,
            )
            resp_data_pull = json_loads(rv_pull.data)
            if resp_data_pull["status"] == "running":
                waiting_for_running = False
            elif resp_data_pull["status"] in [
                "error",
                "terminated",
                "timeout",
            ]:
                self.assertTrue(
                    False,
                    "Process is not running it is %s"
                    % resp_data_pull["status"],
                )
        status_url = resp_data_pull["urls"]["status"].split(URL_PREFIX)[-1]

        # job resumption by running job
        rv2 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=json_dumps(process_chain_2_error),
            content_type="application/json",
        )
        self.assertEqual(rv2.status_code, 404)
        resp_data2 = json_loads(rv2.data)
        self.assertEqual(resp_data2["status"], "error")
        self.assertEqual(
            resp_data2["message"], "Resource is running no restart possible"
        )

    def test_job_resumption_importer(self):
        """Test job resumption with processing_async endpoint and importer"""
        tpl = Template(json_dumps(process_chain_3_importer))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation2@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )
        status_url = resp["urls"]["status"].split(URL_PREFIX)[-1]

        # check if interim results are saved
        resp_data = json_loads(rv.data)
        rv_user_id = resp_data["user_id"]
        rv_resource_id = resp_data["resource_id"]
        interim_dir = os.path.join(
            global_config.GRASS_RESOURCE_DIR,
            rv_user_id,
            "interim",
            rv_resource_id,
        )
        self.assertTrue(
            os.path.isdir(interim_dir),
            "Interim results are not stored in the expected folder",
        )

        # resumption of the job
        rv2 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elev_ned_30m_new", map2="baum"),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv2,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

    def test_job_2_times_resumption_importer(self):
        """Test job 2 times resumption with processing_async endpoint and
        importer
        """
        tpl = Template(json_dumps(process_chain_3_importer))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation2@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )
        status_url = resp["urls"]["status"].split(URL_PREFIX)[-1]

        # first job resumption
        rv2 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elev_ned_30m_new", map2="baum1"),
            content_type="application/json",
        )
        resp2 = self.waitAsyncStatusAssertHTTP(
            rv2,
            headers=self.admin_auth_header,
            http_status=400,
            status="error",
        )
        status_url = resp2["urls"]["status"].split(URL_PREFIX)[-1]

        # check if interim results are saved
        resp_data = json_loads(rv2.data)
        rv_user_id = resp_data["user_id"]
        rv_resource_id = resp_data["resource_id"]
        interim_dir = os.path.join(
            global_config.GRASS_RESOURCE_DIR,
            rv_user_id,
            "interim",
            rv_resource_id,
        )
        self.assertTrue(
            os.path.isdir(interim_dir),
            "Interim results are not stored in the expected folder",
        )

        # second job resumption
        rv3 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elev_ned_30m_new", map2="baum"),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv3,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        self.__class__.resource_user_id = rv_user_id
        self.__class__.resource_resource_id = rv_resource_id

    def compare_stdout(self, resp):
        proc_results = resp["process_results"]
        self.assertIn(
            "sld", proc_results, "'sld' not saved in process results"
        )
        self.assertIn(
            "r_info_map1",
            proc_results,
            "'r_info_map1' not saved in process results",
        )
        self.assertIn(
            "r_info_map2",
            proc_results,
            "'r_info_map2' not saved in process results",
        )
        self.assertIsInstance(
            proc_results["sld"], list, "'sld' result is not of type list"
        )
        self.assertIsInstance(
            proc_results["r_info_map1"],
            dict,
            "'r_info_map1' result is not of type dict",
        )
        self.assertIsInstance(
            proc_results["r_info_map2"],
            dict,
            "'r_info_map2' result is not of type dict",
        )
        r_info_map1 = {
            "cells": "2025000",
            "cols": "1500",
            "datatype": "FCELL",
            "east": "645000",
            "ewres": "10",
            "ncats": "255",
            "north": "228500",
            "nsres": "10",
            "rows": "1350",
            "south": "215000",
            "west": "630000",
        }
        self.assertEqual(
            proc_results["r_info_map1"],
            r_info_map1,
            "'r_info_map1' is not equal to the region",
        )
        r_info_map2 = {
            "cells": "2025000",
            "cols": "1500",
            "datatype": "CELL",
            "east": "645000",
            "ewres": "10",
            "ncats": "0",
            "north": "228500",
            "nsres": "10",
            "rows": "1350",
            "south": "215000",
            "west": "630000",
        }
        self.assertEqual(
            proc_results["r_info_map2"],
            r_info_map2,
            "'r_info_map2' is not equal to the region",
        )

    def test_job_resumption_stdout(self):
        """Test job resumption with processing_async endpoint and stdout"""
        tpl = Template(json_dumps(process_chain_5_stdout))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation2@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )
        status_url = resp["urls"]["status"].split(URL_PREFIX)[-1]

        # check if interim results are saved
        resp_data = json_loads(rv.data)
        rv_user_id = resp_data["user_id"]
        rv_resource_id = resp_data["resource_id"]
        interim_dir = os.path.join(
            global_config.GRASS_RESOURCE_DIR,
            rv_user_id,
            "interim",
            rv_resource_id,
        )
        self.assertTrue(
            os.path.isdir(interim_dir),
            "Interim results are not stored in the expected folder",
        )

        # resumption of the job
        rv2 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp2 = self.waitAsyncStatusAssertHTTP(
            rv2,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        self.compare_stdout(resp2)

    def test_job_2_times_resumption_stdout(self):
        """Test job 2 times resumption with processing_async endpoint and
        stdout
        """
        tpl = Template(json_dumps(process_chain_5_stdout))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation2@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )
        status_url = resp["urls"]["status"].split(URL_PREFIX)[-1]

        # first job resumption
        rv2 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum1"),
            content_type="application/json",
        )
        resp2 = self.waitAsyncStatusAssertHTTP(
            rv2,
            headers=self.admin_auth_header,
            http_status=400,
            status="error",
        )
        status_url = resp2["urls"]["status"].split(URL_PREFIX)[-1]

        # check if interim results are saved
        resp_data = json_loads(rv2.data)
        rv_user_id = resp_data["user_id"]
        rv_resource_id = resp_data["resource_id"]
        interim_dir = os.path.join(
            global_config.GRASS_RESOURCE_DIR,
            rv_user_id,
            "interim",
            rv_resource_id,
        )
        self.assertTrue(
            os.path.isdir(interim_dir),
            "Interim results are not stored in the expected folder",
        )

        # second job resumption
        rv3 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp3 = self.waitAsyncStatusAssertHTTP(
            rv3,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        self.compare_stdout(resp3)

    def test_resource_endpoints(self):
        """Test resource endpoint with iterations"""
        self.assertIsNotNone(self.resource_user_id, "resource_user_id is None")
        self.assertIsNotNone(
            self.resource_resource_id, "resource_resource_id is None"
        )

        # get latest iteration by requesting /resources/{USER}/resource_id-{ID}
        rv1 = self.server.get(
            URL_PREFIX
            + "/resources/%s/%s"
            % (self.resource_user_id, self.resource_resource_id),
            headers=self.admin_auth_header,
        )
        resp_data1 = json_loads(rv1.data)
        self.assertEqual(
            resp_data1["status"],
            "finished",
            "Resource request 1 is not finished",
        )
        self.assertEqual(
            rv1.status_code, 200, "Resource request 1 status code is not 200"
        )
        self.assertEqual(
            resp_data1["iteration"], 3, "Iteration is not 3 (the latest)"
        )

        # get all iteration by requesting /resources/{USER}/{ID}
        rv2 = self.server.get(
            URL_PREFIX
            + "/resources/%s/%s"
            % (
                self.resource_user_id,
                self.resource_resource_id.replace("resource_id-", ""),
            ),
            headers=self.admin_auth_header,
        )
        resp_data2 = json_loads(rv2.data)
        self.assertEqual(
            rv2.status_code, 200, "Resource request 2 status code is not 200"
        )
        self.assertEqual(
            len(resp_data2.keys()), 3, "Not 3 iteration in request"
        )

        # get resource by iteration number requesting /resources/{USER}/{ID}/
        # {ITERATION}
        for iteration in range(1, 4):
            rv3 = self.server.get(
                URL_PREFIX
                + "/resources/%s/%s/%d"
                % (
                    self.resource_user_id,
                    self.resource_resource_id,
                    iteration,
                ),
                headers=self.admin_auth_header,
            )
            resp_data3 = json_loads(rv3.data)
            self.assertEqual(
                rv3.status_code,
                200,
                "Resource request for iteration %d status code is not 200"
                % iteration,
            )
            if iteration != 1:
                self.assertIn(
                    "iteration",
                    resp_data3[str(iteration)],
                    "'iteration' is not in response",
                )
                self.assertEqual(
                    resp_data3[str(iteration)]["iteration"],
                    iteration,
                    "Iteration is not %d" % iteration,
                )
            else:
                self.assertNotIn(
                    "iteration",
                    resp_data3[str(iteration)],
                    "'iteration' is in response",
                )
            del rv3, resp_data3


class JobResumptionProcessingExportTestCase(JobResumptionProcessingTestCase):
    endpoint = "/locations/nc_spm_08/processing_async_export"
    resource_user_id = None
    resource_resource_id = None

    def test_job_resumption_exporter(self):
        """
        Test job resumption with processing_async_export endpoint and exporter
        """
        tpl = Template(json_dumps(process_chain_4_exporter))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation2@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )
        status_url = resp["urls"]["status"].split(URL_PREFIX)[-1]

        # check if interim results are saved
        resp_data = json_loads(rv.data)
        rv_user_id = resp_data["user_id"]
        rv_resource_id = resp_data["resource_id"]
        interim_dir = os.path.join(
            global_config.GRASS_RESOURCE_DIR,
            rv_user_id,
            "interim",
            rv_resource_id,
        )
        self.assertTrue(
            os.path.isdir(interim_dir),
            "Interim results are not stored in the expected folder",
        )

        # resumption of the job
        rv2 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp2 = self.waitAsyncStatusAssertHTTP(
            rv2,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

        # Get the exported results
        urls = resp2["urls"]["resources"]
        for url in urls:
            rv = self.server.get(url, headers=self.admin_auth_header)
            self.assertEqual(
                rv.status_code,
                200,
                "HTML status code is wrong %i" % rv.status_code,
            )
            if url.endswith(".tif"):
                self.assertEqual(
                    rv.mimetype,
                    "image/tiff",
                    "Wrong mimetype %s" % rv.mimetype,
                )
            elif url.endswith(".zip"):
                self.assertEqual(
                    rv.mimetype,
                    "application/zip",
                    "Wrong mimetype %s" % rv.mimetype,
                )

    def test_job_2_times_resumption_exporter(self):
        """Test job 2 times resumption with processing_async_export endpoint
        and exporter
        """
        tpl = Template(json_dumps(process_chain_4_exporter))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation2@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )
        status_url = resp["urls"]["status"].split(URL_PREFIX)[-1]

        # first job resumption
        rv2 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum1"),
            content_type="application/json",
        )
        resp2 = self.waitAsyncStatusAssertHTTP(
            rv2,
            headers=self.admin_auth_header,
            http_status=400,
            status="error",
        )
        status_url = resp2["urls"]["status"].split(URL_PREFIX)[-1]

        # check if interim results are saved
        resp_data = json_loads(rv2.data)
        rv_user_id = resp_data["user_id"]
        rv_resource_id = resp_data["resource_id"]
        interim_dir = os.path.join(
            global_config.GRASS_RESOURCE_DIR,
            rv_user_id,
            "interim",
            rv_resource_id,
        )
        self.assertTrue(
            os.path.isdir(interim_dir),
            "Interim results are not stored in the expected folder",
        )

        # second job resumption
        rv3 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp3 = self.waitAsyncStatusAssertHTTP(
            rv3,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

        # Get the exported results
        urls = resp3["urls"]["resources"]
        for url in urls:
            rv = self.server.get(url, headers=self.admin_auth_header)
            self.assertEqual(
                rv.status_code,
                200,
                "HTML status code is wrong %i" % rv.status_code,
            )
            if url.endswith(".tif"):
                self.assertEqual(
                    rv.mimetype,
                    "image/tiff",
                    "Wrong mimetype %s" % rv.mimetype,
                )
            elif url.endswith(".zip"):
                self.assertEqual(
                    rv.mimetype,
                    "application/zip",
                    "Wrong mimetype %s" % rv.mimetype,
                )
        self.__class__.resource_user_id = rv_user_id
        self.__class__.resource_resource_id = rv_resource_id


class JobResumptionPersistentProcessingTestCase(
    JobResumptionProcessingTestCase
):
    location = "nc_spm_08"
    mapset = "test_mapset"
    endpoint = "/locations/%s/mapsets/%s/processing_async" % (location, mapset)
    resource_user_id = None
    resource_resource_id = None
    mapset_created = True

    def tearDown(self):
        if self.mapset_created is True:
            rv = self.server.delete(
                URL_PREFIX
                + "/locations/%s/mapsets/%s/lock"
                % (self.location, self.mapset),
                headers=self.admin_auth_header,
            )
            self.waitAsyncStatusAssertHTTP(rv, headers=self.admin_auth_header)
            rv2 = self.server.delete(
                URL_PREFIX
                + "/locations/%s/mapsets/%s" % (self.location, self.mapset),
                headers=self.admin_auth_header,
            )
            self.waitAsyncStatusAssertHTTP(rv2, headers=self.admin_auth_header)
        else:
            self.__class__.mapset_created = True
        self.app_context.pop()

    def test_saved_interim_results(self):
        """Test if the interim results are removed"""
        self.create_new_mapset(self.mapset, self.location)
        tpl = Template(json_dumps(process_chain_1))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

        # check if interim results are saved
        resp_data = json_loads(rv.data)
        rv_user_id = resp_data["user_id"]
        rv_resource_id = resp_data["resource_id"]
        interim_dir = os.path.join(
            global_config.GRASS_RESOURCE_DIR,
            rv_user_id,
            "interim",
            rv_resource_id,
        )
        self.assertTrue(
            not os.path.isdir(interim_dir),
            "Interim results are not stored in the expected folder",
        )

    def test_job_resumption_error_by_running(self):
        super(
            JobResumptionPersistentProcessingTestCase, self
        ).test_job_resumption_error_by_running()
        sleep(self.sleep_time)
        self.__class__.mapset_created = False
        print(self.resource_user_id)

    def test_resource_endpoints(self):
        super(
            JobResumptionPersistentProcessingTestCase, self
        ).test_resource_endpoints()
        self.__class__.mapset_created = False


class JobResumptionErrorTestCase(ActiniaResourceTestCaseBase):
    endpoint = "/locations/nc_spm_08/processing_async"

    def test_job_resumption_config_error(self):
        """Test if the job resumption fails if save_interim_results is set to
        False in the actinia.cfg
        """
        tpl = Template(json_dumps(process_chain_1))
        rv = self.server.post(
            URL_PREFIX + self.endpoint,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation2@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv, headers=self.admin_auth_header, http_status=400, status="error"
        )
        status_url = resp["urls"]["status"].split(URL_PREFIX)[-1]
        rv2 = self.server.put(
            URL_PREFIX + status_url,
            headers=self.admin_auth_header,
            data=tpl.render(map1="elevation@PERMANENT", map2="baum"),
            content_type="application/json",
        )
        self.assertEqual(rv2.status_code, 404)
        resp_data = json_loads(rv2.data)
        self.assertEqual(resp_data["status"], "error")
        self.assertEqual(
            resp_data["message"],
            "Interim results are not set in the configuration",
        )


class JobResumptionProcessingTestCaseOnError(JobResumptionProcessingTestCase):
    save_interim_results = "onError"


class JobResumptionProcessingExportTestCaseOnError(
    JobResumptionProcessingExportTestCase
):
    save_interim_results = "onError"


class JobResumptionPersistentProcessingTestCaseOnError(
    JobResumptionPersistentProcessingTestCase
):
    save_interim_results = "onError"


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Download cache management

TODO: Tests required
"""

from flask import jsonify, make_response
from flask_restful_swagger_2 import swagger
import pickle
from actinia_api.swagger2.actinia_core.apidocs import download_cache_management

from actinia_core.rest.base.resource_base import ResourceBase
from actinia_core.core.common.redis_interface import enqueue_job
from actinia_core.core.common.api_logger import log_api_call
from actinia_core.rest.base.endpoint_config import (
    check_endpoint,
    endpoint_decorator,
)
from actinia_core.rest.base.user_auth import check_admin_role
from actinia_core.rest.base.user_auth import check_user_permissions
from actinia_core.core.common.app import auth
from actinia_core.processing.common.download_cache_management import (
    start_download_cache_size,
    start_download_cache_remove,
)

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


class SyncDownloadCacheResource(ResourceBase):
    decorators = [
        log_api_call,
        check_user_permissions,
        check_admin_role,
        auth.login_required,
    ]

    @endpoint_decorator()
    @swagger.doc(check_endpoint("get", download_cache_management.get_doc))
    def get(self):
        """Get the current size of the download cache"""
        rdc = self.preprocess(has_json=False, has_xml=False)

        if rdc:
            enqueue_job(
                self.job_timeout,
                start_download_cache_size,
                rdc,
                queue_type_overwrite=True,
            )
            http_code, response_model = self.wait_until_finish()
        else:
            http_code, response_model = pickle.loads(self.response_data)

        return make_response(jsonify(response_model), http_code)

    @endpoint_decorator()
    @swagger.doc(
        check_endpoint("delete", download_cache_management.delete_doc)
    )
    def delete(self):
        """Clean the download cache and remove all cached data"""
        rdc = self.preprocess(has_json=False, has_xml=False)

        if rdc:
            enqueue_job(
                self.job_timeout,
                start_download_cache_remove,
                rdc,
                queue_type_overwrite=True,
            )
            http_code, response_model = self.wait_until_finish()
        else:
            http_code, response_model = pickle.loads(self.response_data)

        return make_response(jsonify(response_model), http_code)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Actinia Core REST API call logging
"""
from datetime import datetime
import pickle
from functools import wraps
from flask import g, abort, request
import platform
from actinia_core.core.redis_api_log import redis_api_log_interface
from actinia_core.core.redis_fluentd_logger_base import RedisFluentLoggerBase

try:
    from fluent import sender
    from fluent import event

    has_fluent = True
except Exception:
    has_fluent = False

__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-present, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"


def log_api_call(f):
    """This decorator function logs API calls

    It stores the request information, user and host id in a
    database list identified by the user name

    Args:
        f (func): The function to wrap

    Returns:
        func:
        The function

    """

    @wraps(f)
    def decorated_function(*args, **kwargs):
        if g.user is None:
            abort(401)

        logger = ApiLogger()
        user_id = g.user.get_id()

        logger.add_entry(user_id=user_id, http_request=request)

        return f(*args, **kwargs)

    return decorated_function


class ApiLogger(RedisFluentLoggerBase):
    db = redis_api_log_interface

    def __init__(self, config=None, user_id=None, fluent_sender=None):
        RedisFluentLoggerBase.__init__(
            self, config=config, user_id=user_id, fluent_sender=fluent_sender
        )

    def add_entry(self, user_id, http_request):
        """Add an API call entry to the database

        Args:
            user_id (str): The user id of the API log
            http_request: The http request object


            example = {
                "endpoint": "asyncephemeralresource",
                "method": "POST",
                "path": "/locations/nc_spm_08/processing_async",
                "request_url": "http://localhost/locations/nc_spm_08/"
                "processing_async"
              }


        Returns:
            int:
            The index of the new entry in the api log list

        """
        api_info = {
            "endpoint": http_request.endpoint,
            "method": http_request.method,
            "path": http_request.path,
            "request_url": http_request.url,
        }

        entry = {
            "time_stamp": datetime.now(),
            "node": platform.node(),
            "api_info": api_info,
            "request_str": str(http_request),
            "user_id": user_id,
            "status": "api_call",
            "logger": "api_logger",
        }

        # Serialize the entry
        pentry = pickle.dumps(entry)

        redis_return = bool(self.db.add(user_id, pentry))

        entry["time_stamp"] = str(entry["time_stamp"])
        self.send_to_logger("API_LOG", entry)
        return redis_return

    def list(self, user_id, start, end):
        """
        Return a list of api log entries

        Args:
            user_id (str): The user id of the API log
            start (int): Integer start index
            end (int): Integer end index

        Returns:
            list:
            A list of log entries

        Example:

            [{"time_stamp":"...",
             "node":"...",
             "request":"..."},
             {..},]

        """

        l_entries = self.db.list(user_id, start, end)

        # We need to deserialize the log entries
        result_list = []

        for pentry in l_entries:
            entry = pickle.loads(pentry)
            result_list.append(entry)

        return result_list

    def trim(self, user_id, start, end):
        """Remove all api log entries that are outside the specified indices

        Args:
            user_id (str): The user_id to trim the log entries
            start (int): The start index
            end (int): The end index

        Returns:
            bool:
            True in any case

        """
        return bool(self.db.trim(user_id, start, end))

    def delete(self, user_id):
        """Delete the user_id specific api log list

        Args:
            user_id (str): The user_id to delete the log entries

        Returns:
            bool:
            True in case of success, False otherwise

        """

        return bool(self.db.delete(user_id))

    def size(self, user_id):
        """Return the size of the user_id specific log list
        Args:
            user_id (str): The user_id

        Returns:
            int:
            The size of the user specific log list

        """

        return self.db.size(user_id)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Redis server lock interface
"""

import redis

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"
__credits__ = [
    "Thünen Institutes of Climate-Smart Agriculture",
    "https://www.ti.bund.de/en/ak/",
]


class RedisLockingInterface(object):
    """
    The Redis locking database interface
    """

    # Redis LUA script to lock e resource
    # Two keys must be provided, the name of the resource and the expiration
    # time in seconds
    # lock_resource("location/mapset", 30)
    # Return 1 for success and 0 for unable to acquire lock because
    # resource-lock already exists
    lua_lock_resource = """
    local value_exists = redis.call('EXISTS', KEYS[1])
    if value_exists == 0 then
      redis.call("SETEX", KEYS[1], KEYS[2], 1)
      return 1
    end
    return 0
    """

    # LUA script to extend the lock valid time
    # Two keys must be provided, the name of the resource and the expiration
    # time in seconds
    # extend_resource_lock("user/location/mapset", 30)
    # Return 1 for success, 0 for resource does not exists
    lua_extend_resource_lock = """
    local value_exists = redis.call('EXISTS', KEYS[1])
    if value_exists == 0 then
        return 0
    else
      redis.call("EXPIRE", KEYS[1], KEYS[2])
      return 1
    end
    """

    # LUA script to unlock a resource
    # Return 1 for success, 0 for resource does not exists
    lua_unlock_resource = """
    local value_exists = redis.call('EXISTS', KEYS[1])
    if value_exists == 0 then
        return 0
    else
      redis.call("DEL", KEYS[1])
      return 1
    end
    """

    # Locks are Key-Value pairs in the Redis database using SET and DEl for
    # management
    lock_prefix = "RESOURCE-LOCK::"

    def __init__(self):
        self.connection_pool = None
        self.redis_server = None
        self.call_lock_resource = None
        self.call_extend_resource_lock = None
        self.call_unlock_resource = None

    def connect(self, host, port, password=None):
        """Connect to a specific redis server

        Args:
            host (str): The host name or IP address
            port (int): The port
            password (str): The password

        """
        kwargs = dict()
        kwargs["host"] = host
        kwargs["port"] = port
        if password and password is not None:
            kwargs["password"] = password
        self.connection_pool = redis.ConnectionPool(**kwargs)
        del kwargs
        self.redis_server = redis.StrictRedis(
            connection_pool=self.connection_pool
        )

        # Register the resource lock scripts in Redis
        self.call_lock_resource = self.redis_server.register_script(
            self.lua_lock_resource
        )
        self.call_extend_resource_lock = self.redis_server.register_script(
            self.lua_extend_resource_lock
        )
        self.call_unlock_resource = self.redis_server.register_script(
            self.lua_unlock_resource
        )

    def disconnect(self):
        self.connection_pool.disconnect()

    """
    LOCK
    The lock mechanism can be used to avoid concurrent access to GRASS GIS
    mapsets by several processes. A mapset has a unique id:

        location/mapset

    That can be used to create a resource lock. Locking and lock checking
    are atomic operations. Hence, it is guaranteed that only a single
    process in a distributed system can access the resource if the locking
    mechanism is used.
    """

    def get(self, resource_id, expiration=30):
        """Get the status of a resource lock

        Args:
            resource_id (str): Name of the resource to lock, for example
                               "location/mapset"

        Returns:
             bool:
             True if resource is locked, False otherwise

        """
        return bool(self.redis_server.get(self.lock_prefix + str(resource_id)))

    def lock(self, resource_id, expiration=30):
        """Lock a resource for a specific time frame

        The lock is acquired for the provided time

        This function will put a prefix before the resource name
        to avoid key conflicts with other resources that are logged
        in the Redis database.

        Args:
            resource_id (str): Name of the resource to lock, for example
                               "location/mapset"
            expiration (int): The time in seconds for which the lock is
                              acquired

        Returns:
             int:
             1 for success and 0 if unable to acquire lock because
             resource-lock already exists

        """

        keys = [self.lock_prefix + str(resource_id), expiration]
        return self.call_lock_resource(keys=keys)

    def extend(self, resource_id, expiration=30):
        """Extent the expiration of a resource lock for a specific time frame

        This function will put a prefix before the resource name
        to avoid key conflicts with other resources that are logged
        in the Redis database.

        Args:
            resource_id (str): Name of the resource to extent the lock, for
                               example "location/mapset"
            expiration (int): The time in seconds for which the lock is
                              acquired

        Returns:
            int:
            1 for success and 0 if unable to extent the lock because resource
            does not exists

        """
        keys = [self.lock_prefix + str(resource_id), expiration]
        # print("Extend Lock", expiration, self.lock_prefix + str(resource_id),
        # str(self))
        return self.call_extend_resource_lock(keys=keys)

    def unlock(self, resource_id):
        """Unlock a resource

        This function will put a prefix before the resource name
        to avoid key conflicts with other resources that are logged
        in the Redis database.

        Args:
            resource_id (str): Name of the resource to remove the lock, for
                               example "location/mapset"

        Returns:
            int:
            1 for success and 0 if unable to unlock

        """
        keys = [
            self.lock_prefix + str(resource_id),
        ]
        # print("UnLock", self.lock_prefix + str(resource_id), str(self))
        return self.call_unlock_resource(keys=keys)


# Create the Redis interface instance
# redis_lock_interface = RedisLockingInterface()


def test_locking(r):
    resource = "location/mapset"

    # Remove the lock if its present
    r.unlock(resource)

    ret = r.lock(resource, 5)
    if ret != 1:
        raise Exception("lock_resource does not work")

    ret = r.get(resource, 5)
    print("Lock status True", ret)
    if ret is not True:
        raise Exception("get lock does not work")

    ret = r.get("nothing", 5)
    print("Lock status False", ret)
    if ret is not False:
        raise Exception("get lock does not work")

    ret = r.extend(resource, 5)
    if ret != 1:
        raise Exception("extend_resource_lock does not work")
    ret = r.lock(resource, 5)
    if ret != 0:
        raise Exception("lock_resource does not work")
    ret = r.unlock(resource)
    if ret != 1:
        raise Exception("unlock_resource does not work")
    ret = r.unlock(resource)
    if ret != 0:
        raise Exception("unlock_resource does not work")
    ret = r.extend(resource, 5)
    if ret != 0:
        raise Exception("extend_resource_lock does not work")


if __name__ == "__main__":
    import os
    import signal
    import time

    pid = os.spawnl(
        os.P_NOWAIT, "/usr/bin/redis-server", "./redis.conf", "--port 7000"
    )

    time.sleep(1)

    try:
        r = RedisLockingInterface()
        r.connect(host="localhost", port=7000)
        test_locking(r)
        r.disconnect()
    except Exception:
        raise
    finally:
        os.kill(pid, signal.SIGTERM)



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: STRDS test case
"""
from flask.json import loads as json_loads, dumps as json_dumps
import unittest

try:
    from .test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
    )
except ModuleNotFoundError:
    from test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
    )

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"

location = "nc_spm_08"
strds_mapset = "modis_lst"
strds_url = URL_PREFIX + "/locations/%(location)s/mapsets/%(mapset)s/strds" % {
    "location": location,
    "mapset": strds_mapset,
}
strds_data = "LST_Day_monthly"


class STRDSTestCase(ActiniaResourceTestCaseBase):
    def test_list_strds(self):
        rv = self.server.get(strds_url, headers=self.user_auth_header)
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        strds_list = json_loads(rv.data)["process_results"]
        self.assertTrue("LST_Day_monthly" in strds_list)

    def test_list_strds_where_1(self):
        rv = self.server.get(
            strds_url + "?where=start_time == '2015-01-01 00:00:00'",
            headers=self.user_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        strds_list = json_loads(rv.data)["process_results"]
        self.assertTrue("LST_Day_monthly" in strds_list)

    def test_list_strds_where_2(self):
        rv = self.server.get(
            strds_url + "?where=start_time > '2016-01-01'",
            headers=self.user_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        strds_list = json_loads(rv.data)["process_results"]

        self.assertEqual(len(strds_list), 0)

    def test_strds_info(self):
        rv = self.server.get(
            strds_url + "/%s" % strds_data, headers=self.user_auth_header
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        start_time = json_loads(rv.data)["process_results"]["start_time"]

        self.assertEqual(start_time, "'2015-01-01 00:00:00'")

    def test_strds_create_remove(self):
        new_mapset = "strds_test"
        self.create_new_mapset(mapset_name=new_mapset, location_name=location)

        # Create success
        rv = self.server.post(
            URL_PREFIX
            + "/locations/%s/mapsets/%s/strds/test_strds"
            % (location, new_mapset),
            headers=self.admin_auth_header,
            data=json_dumps(
                {
                    "temporaltype": "absolute",
                    "title": "A nice title",
                    "description": "A nice description",
                }
            ),
            content_type="application/json",
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Create failure since the strds already exists
        rv = self.server.post(
            URL_PREFIX
            + "/locations/%s/mapsets/%s/strds/test_strds"
            % (location, new_mapset),
            headers=self.admin_auth_header,
            data=json_dumps(
                {
                    "temporaltype": "absolute",
                    "title": "A nice title",
                    "description": "A nice description",
                }
            ),
            content_type="application/json",
        )
        # print(rv.data)
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )
        # Read/check information of the new strds
        rv = self.server.get(
            URL_PREFIX
            + "/locations/%s/mapsets/%s/strds/test_strds"
            % (location, new_mapset),
            headers=self.user_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        start_time = json_loads(rv.data)["process_results"]["start_time"]

        self.assertEqual(start_time, "'None'")
        # Delete the strds
        rv = self.server.delete(
            URL_PREFIX
            + "/locations/%s/mapsets/%s/strds/test_strds"
            % (location, new_mapset),
            headers=self.admin_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )
        # Try to delete the strds again to produce an error
        rv = self.server.delete(
            URL_PREFIX
            + "/locations/%s/mapsets/%s/strds/test_strds"
            % (location, new_mapset),
            headers=self.admin_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        rv = self.server.get(
            URL_PREFIX
            + "/locations/%s/mapsets/%s/strds/test_strds"
            % (location, new_mapset),
            headers=self.user_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_strds_info_error_1(self):
        # Raster does not exist
        rv = self.server.get(
            strds_url + "/precipitation_1950_2013_yearly_mm_nope",
            headers=self.user_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_list_strds_where_error_1(self):
        # Wrong where statement
        rv = self.server.get(
            strds_url + "?where=start_timing > '2000-01-01'",
            headers=self.user_auth_header,
        )
        print(rv.data)
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Keycloak user management

Possible TODOs: add list_all_users, delete, create_user (, exists).
        In this case, keycloak admin console would be needed and actinia would
        need to store keycloak admin credentials!
"""
from keycloak import KeycloakOpenID
from jose.exceptions import ExpiredSignatureError

from actinia_core.core.common.user_base import (
    ActiniaUserBase,
)
from actinia_core.core.logging_interface import log
from actinia_core.core.common.config import global_config

__author__ = "Anika Weinmann"
__copyright__ = "Copyright 2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


def create_user_from_tokeninfo(token_info):
    """
    Function to create a keycloak user from the keycloak token.
    """
    attr_prefix = (
        global_config.KEYCLOAK_ATTR_PREFIX
        if global_config.KEYCLOAK_ATTR_PREFIX is not None
        else ""
    )
    user_id = token_info["preferred_username"]
    kwargs = {
        "user_role": token_info["resource_access"][
            global_config.KEYCLOAK_CLIENT_ID
        ]["roles"][0],
        "cell_limit": token_info[f"{attr_prefix}cell_limit"],
        "process_num_limit": token_info[f"{attr_prefix}process_num_limit"],
        "process_time_limit": token_info[f"{attr_prefix}process_time_limit"],
    }
    acc_ds_name = f"{attr_prefix}accessible_datasets"
    if token_info[acc_ds_name] and token_info[acc_ds_name] != "None":
        kwargs["accessible_datasets"] = token_info[acc_ds_name]
    acc_mod_name = f"{attr_prefix}accessible_modules"
    if token_info[acc_mod_name] and token_info[acc_mod_name] != "None":
        kwargs["accessible_modules"] = token_info[acc_mod_name]
    groups = list()
    for group in token_info["groups"]:
        if group.startswith(global_config.KEYCLOAK_GROUP_PREFIX):
            groups.append(
                group.replace(global_config.KEYCLOAK_GROUP_PREFIX, "")
            )
    if len(groups) == 0:
        log.error(f"Authentication ERROR: User {user_id} has no user group.")
        return None
    elif len(groups) > 1:
        log.warning(
            f"User {user_id} has more than one group, taking {groups[0]}."
        )
    kwargs["user_group"] = groups[0]
    user = ActiniaKeycloakUser(
        user_id,
        **kwargs,
    )
    user._generate_permission_dict()
    # adding group members
    if "group_members" in token_info and token_info["group_members"]:
        user.set_group_members(
            ",".join(token_info["group_members"]).split(",")
        )
    return user


class ActiniaKeycloakUser(ActiniaUserBase):
    """
    The Actinia Core keycloak user management class

    This class manages a user which is stored in keycloak
    """

    def __init__(
        self,
        user_id,
        user_group=None,
        user_role=None,
        accessible_datasets={
            "nc_spm_08": ["PERMANENT", "user1", "landsat"],
            "ECAD": ["PERMANENT"],
            "latlong_wgs84": ["PERMANENT"],
        },
        accessible_modules=global_config.MODULE_ALLOW_LIST,
        cell_limit=global_config.MAX_CELL_LIMIT,
        process_num_limit=global_config.PROCESS_NUM_LIMIT,
        process_time_limit=global_config.PROCESS_TIME_LIMT,
    ):
        if isinstance(accessible_datasets, str):
            datasets = dict()
            lm_list = accessible_datasets.split(",")
            for lm in lm_list:
                if "/" in lm:
                    location, mapset = lm.split("/")
                else:
                    location = lm
                    mapset = None
                if location not in datasets:
                    datasets[location] = []
                datasets[location].append(mapset)
        else:
            datasets = accessible_datasets
        if isinstance(accessible_modules, str):
            modules = accessible_modules.split(",")
        else:
            modules = accessible_modules
        super().__init__(
            user_id,
            user_group=user_group,
            user_role=user_role,
            accessible_datasets=datasets,
            accessible_modules=modules,
            cell_limit=cell_limit,
            process_num_limit=process_num_limit,
            process_time_limit=process_time_limit,
        )
        self.group_members = list()

    @staticmethod
    def verify_keycloak_token(token):
        keycloak_openid = KeycloakOpenID(
            server_url=global_config.KEYCLOAK_URL,
            client_id=global_config.KEYCLOAK_CLIENT_ID,
            realm_name=global_config.KEYCLOAK_REALM,
            client_secret_key=global_config.KEYCLOAK_CLIENT_SECRET_KEY,
        )
        KEYCLOAK_PUBLIC_KEY = (
            "-----BEGIN PUBLIC KEY-----\n"
            + keycloak_openid.public_key()
            + "\n-----END PUBLIC KEY-----"
        )
        options = {
            "verify_signature": True,
            "verify_aud": True,
            "verify_exp": True,
        }
        try:
            token_info = keycloak_openid.decode_token(
                token, key=KEYCLOAK_PUBLIC_KEY, options=options
            )
        except ExpiredSignatureError:
            return None
        except Exception:
            return None
        return create_user_from_tokeninfo(token_info)

    def set_group_members(self, group_members):
        """Set the user group_members

        Args:
            group: The user group_members

        """
        self.group_members = group_members

    def check_group_members(self, user_id):
        """Check if the user_id is in the group_members attribute.

        Args:
            user_id (str): The id (name, email, ..) of the user that must be
                           unique

        Returns:
            bool:
            Return the if the user_id is in the group_members
        """
        return user_id in self.group_members

    # def exists(self):  # rest/user_management ???
    #     """Check if the user exists
    #
    #     Returns:
    #         bool:
    #         True if the user exists, False otherwise
    #     """
    #     if self.user_id is None:
    #         return False
    #
    #     return self.db.exists(self.user_id)

    def get_role(self):
        """Return the role

        Returns:
            str:
            Return the role
        """
        return self.user_role

    def get_group(self):
        """Return the user group

        Returns:
            str:
            Return the user group
        """
        return self.user_group

    def get_credentials(self):
        """Return the user credentials as a dictionary

        Returns:
            dict:
            Return the user credentials as a dictionary
        """
        self._generate_permission_dict()
        credentials = {
            "user_id": self.user_id,
            "user_role": self.user_role,
            "user_group": self.user_group,
            "permissions": self.permissions,
        }
        return credentials

    def get_accessible_datasets(self):
        """Return a dictionary of location:mapset list entries

        Returns:
            dict:
            Return a dictionary of location:mapset list entries
        """
        self._generate_permission_dict()

        if self.permissions and "accessible_datasets" in self.permissions:
            return self.permissions["accessible_datasets"]

    def get_accessible_modules(self):
        """Return a list of all accessible modules

        Returns:
            list:
            Return a list of all accessible modules
        """
        self._generate_permission_dict()

        if self.permissions and "accessible_modules" in self.permissions:
            return self.permissions["accessible_modules"]

    def get_cell_limit(self):
        """Return the cell limit

        Returns:
            int:
            The value or None if nothing was found
        """
        self._generate_permission_dict()

        if self.permissions and "cell_limit" in self.permissions:
            return self.permissions["cell_limit"]

    def get_process_num_limit(self):
        """Return the process number limit

        Returns:
            int:
            The value or None if nothing was found
        """
        self._generate_permission_dict()

        if self.permissions and "process_num_limit" in self.permissions:
            return self.permissions["process_num_limit"]

    def get_process_time_limit(self):
        """Return the process time limit

        Returns:
            int:
            The value or None if nothing was found
        """
        self._generate_permission_dict()

        if self.permissions and "process_time_limit" in self.permissions:
            return self.permissions["process_time_limit"]



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: Raster layer test
"""
import unittest
from pprint import pprint
from flask.json import loads as json_load
from flask.json import dumps as json_dumps

try:
    from .test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX
except ModuleNotFoundError:
    from test_resource_base import ActiniaResourceTestCaseBase, URL_PREFIX


__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Soeren Gebbert"
__email__ = "soerengebbert@googlemail.com"


class RasterLayerTestCase(ActiniaResourceTestCaseBase):
    def test_raster_layer_get_colors(self):
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/raster_layers"
            "/elevation/colors",
            headers=self.user_auth_header,
        )
        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        colors = json_load(rv.data)["process_results"]
        self.assertTrue(len(colors) == 8)

    def test_raster_layer_set_colors(self):
        new_mapset = "raster_test_mapset"
        self.create_new_mapset(new_mapset)

        # Create
        postbody = {
            "list": [
                {
                    "id": "set_region",
                    "module": "g.region",
                    "inputs": [
                        {"param": "n", "value": "228500"},
                        {"param": "s", "value": "215000"},
                        {"param": "e", "value": "645000"},
                        {"param": "w", "value": "630000"},
                        {"param": "ewres", "value": "50"},
                        {"param": "nsres", "value": "50"},
                    ],
                },
                {
                    "id": "create_raster",
                    "module": "r.mapcalc",
                    "inputs": [
                        {"param": "expression", "value": "test_layer = 1"}
                    ],
                },
            ],
            "version": "1",
        }
        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/processing_async" % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(postbody),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.user_auth_header,
            http_status=200,
            status="finished",
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        rules = {
            "rules": [
                "1 0:0:0",
            ]
        }

        # Set the color table
        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/test_layer/colors"
            % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(rules),
            content_type="application/json",
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        rules = {"color": "elevation"}

        # Set the color table
        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/test_layer/colors"
            % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(rules),
            content_type="application/json",
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        rules = {"raster": "elevation@PERMANENT"}

        # Set the color table
        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/test_layer/colors"
            % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(rules),
            content_type="application/json",
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Delete
        rv = self.server.delete(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/test_layer"
            % new_mapset,
            headers=self.user_auth_header,
        )
        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_1_raster_layer_set_colors_errors(self):
        new_mapset = "raster_test_mapset"
        self.create_new_mapset(new_mapset)

        # Create
        postbody = {
            "list": [
                {
                    "id": "set_region",
                    "module": "g.region",
                    "inputs": [
                        {"param": "n", "value": "228500"},
                        {"param": "s", "value": "215000"},
                        {"param": "e", "value": "645000"},
                        {"param": "w", "value": "630000"},
                        {"param": "ewres", "value": "50"},
                        {"param": "nsres", "value": "50"},
                    ],
                },
                {
                    "id": "create_raster",
                    "module": "r.mapcalc",
                    "inputs": [
                        {"param": "expression", "value": "test_layer = 1"}
                    ],
                },
            ],
            "version": "1",
        }
        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/processing_async" % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(postbody),
            content_type="application/json",
        )
        self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )
        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        rules = {
            "rules": [
                "wrong format",
            ]
        }

        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/test_layer/colors"
            % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(rules),
            content_type="application/json",
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        log = json_load(rv.data)["message"]
        self.assertTrue("AsyncProcessError:" in log)

        rules = {"rules": "blub"}

        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/test_layer/colors"
            % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(rules),
            content_type="application/json",
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Two rules
        rules = {"color": "elevation", "raster": "elevation@PERMANENT"}

        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/test_layer/colors"
            % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(rules),
            content_type="application/json",
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Wrong format
        rules = {"nonsense": "bla"}

        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/test_layer/colors"
            % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(rules),
            content_type="application/json",
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Wrong format
        rules = [1, 2, 3]

        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/test_layer/colors"
            % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(rules),
            content_type="application/json",
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Raster layer not found
        rules = {"raster": "elevation_nope@PERMANENT"}

        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/test_layer/colors"
            % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(rules),
            content_type="application/json",
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        log = json_load(rv.data)["message"]
        self.assertTrue("AsyncProcessError:" in log)

        # No mapset in name
        rules = {"raster": "elevation"}

        rv = self.server.post(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/test_layer/colors"
            % new_mapset,
            headers=self.user_auth_header,
            data=json_dumps(rules),
            content_type="application/json",
        )

        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        # Delete
        rv = self.server.delete(
            URL_PREFIX
            + "/locations/nc_spm_08/mapsets/%s/raster_layers/test_layer"
            % new_mapset,
            headers=self.user_auth_header,
        )
        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            200,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

    def test_raster_layer_colors_error_1(self):
        # Raster does not exist
        rv = self.server.get(
            f"{URL_PREFIX}/locations/nc_spm_08/mapsets/PERMANENT/raster_layers"
            "/elevat/colors",
            headers=self.user_auth_header,
        )
        pprint(json_load(rv.data))
        self.assertEqual(
            rv.status_code,
            400,
            "HTML status code is wrong %i" % rv.status_code,
        )
        self.assertEqual(
            rv.mimetype, "application/json", "Wrong mimetype %s" % rv.mimetype
        )

        log = json_load(rv.data)["message"]
        self.assertTrue("AsyncProcessError:" in log)
        self.assertTrue("Error while running executable <r.colors.out" in log)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Tests: AsyncProcess test case
"""
import unittest
from flask.json import dumps as json_dumps

try:
    from .test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
        additional_external_data,
    )
except ModuleNotFoundError:
    from test_resource_base import (
        ActiniaResourceTestCaseBase,
        URL_PREFIX,
        additional_external_data,
    )

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2018, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "Sören Gebbert"
__email__ = "soerengebbert@googlemail.com"

vector_layer_export = {
    "list": [
        {
            "id": "exporter_1",
            "outputs": [
                {
                    "export": {"type": "vector", "format": "GML"},
                    "param": "map",
                    "value": "hospitals",
                },
                {
                    "export": {"type": "vector", "format": "GeoJSON"},
                    "param": "map",
                    "value": "nc_state",
                },
                {
                    "export": {"type": "vector", "format": "GPKG"},
                    "param": "map",
                    "value": "nc_state",
                },
                {
                    "export": {"type": "vector", "format": "SQLite"},
                    "param": "map",
                    "value": "nc_state",
                },
                {
                    "export": {"type": "vector", "format": "ESRI_Shapefile"},
                    "param": "map",
                    "value": "firestations",
                },
                {
                    "export": {"type": "vector", "format": "CSV"},
                    "param": "map",
                    "value": "poi_names_wake",
                },
            ],
            "module": "exporter",
        }
    ],
    "version": "1",
}


vector_layer_buffer = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data["rio_json"],
                        "type": "vector",
                    },
                    "param": "map",
                    "value": "input_point",
                }
            ],
        },
        {
            "id": "v_buffer",
            "module": "v.buffer",
            "inputs": [
                {"param": "input", "value": "input_point"},
                {"param": "output", "value": "buf_point"},
                {"param": "distance", "value": "0.001"},
            ],
        },
        {
            "id": "exporter_1",
            "module": "exporter",
            "outputs": [
                {
                    "export": {"type": "vector", "format": "GeoJSON"},
                    "param": "map",
                    "value": "buf_point",
                }
            ],
        },
    ],
    "version": "1",
}


vector_layer_clean = {
    "list": [
        {
            "id": "importer_1",
            "module": "importer",
            "inputs": [
                {
                    "import_descr": {
                        "source": additional_external_data["rio_json"],
                        "type": "vector",
                    },
                    "param": "map",
                    "value": "map_to_clean",
                }
            ],
        },
        {
            "id": "v_clean",
            "module": "v.clean",
            "inputs": [
                {"param": "input", "value": "map_to_clean"},
                {"param": "output", "value": "map_cleaned"},
                {"param": "type", "value": "area"},
                {"param": "tool", "value": "rmarea"},
                {"param": "threshold", "value": "0.0000000001"},
            ],
        },
        {
            "id": "exporter_1",
            "module": "exporter",
            "outputs": [
                {
                    "export": {"type": "vector", "format": "GeoJSON"},
                    "param": "map",
                    "value": "map_cleaned",
                }
            ],
        },
    ],
    "version": "1",
}


class AsyncProcessTestCase(ActiniaResourceTestCaseBase):
    def test_vector_export(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/nc_spm_08/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(vector_layer_export),
            content_type="application/json",
        )

        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

        # Get the exported results
        urls = resp["urls"]["resources"]

        for url in urls:
            print(url)
            rv = self.server.get(url, headers=self.user_auth_header)
            self.assertEqual(
                rv.status_code,
                200,
                "HTML status code is wrong %i" % rv.status_code,
            )
            self.assertEqual(
                rv.mimetype,
                "application/zip",
                "Wrong mimetype %s" % rv.mimetype,
            )

    def test_vector_buffer(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/latlong_wgs84/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(vector_layer_buffer),
            content_type="application/json",
        )
        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

        # Get the exported results
        urls = resp["urls"]["resources"]

        for url in urls:
            print(url)
            rv = self.server.get(url, headers=self.user_auth_header)
            self.assertEqual(
                rv.status_code,
                200,
                "HTML status code is wrong %i" % rv.status_code,
            )
            self.assertEqual(
                rv.mimetype,
                "application/zip",
                "Wrong mimetype %s" % rv.mimetype,
            )

            out_file = open("/tmp/bla.zip", "wb")
            out_file.write(rv.data)
            out_file.close()

    def test_vector_clean(self):
        rv = self.server.post(
            URL_PREFIX + "/locations/latlong_wgs84/processing_async_export",
            headers=self.admin_auth_header,
            data=json_dumps(vector_layer_clean),
            content_type="application/json",
        )

        resp = self.waitAsyncStatusAssertHTTP(
            rv,
            headers=self.admin_auth_header,
            http_status=200,
            status="finished",
        )

        # Get the exported results
        urls = resp["urls"]["resources"]

        for url in urls:
            print(url)
            rv = self.server.get(url, headers=self.user_auth_header)
            self.assertEqual(
                rv.status_code,
                200,
                "HTML status code is wrong %i" % rv.status_code,
            )
            self.assertEqual(
                rv.mimetype,
                "application/zip",
                "Wrong mimetype %s" % rv.mimetype,
            )

            out_file = open("/tmp/bla.zip", "wb")
            out_file.write(rv.data)
            out_file.close()


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2018 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Logging interface
"""

import logging
import logging.handlers
import os
import sys
import platform
from datetime import datetime

from pythonjsonlogger import jsonlogger

from actinia_core.core.common.config import global_config

# unfortunately, the config is read twice because of the read call here but it
# is needed to load the correct interface and log level at this time
global_config.read()

if "colored" in [
    global_config.LOG_STDOUT_FORMAT,
    global_config.LOG_FILE_FORMAT,
]:
    from colorlog import ColoredFormatter

if global_config.LOG_INTERFACE == "fluentd":
    from fluent import handler


__license__ = "GPLv3"
__author__ = "Sören Gebbert, Carmen Tawalika"
__copyright__ = (
    "Copyright 2016-present, Sören Gebbert and mundialis GmbH & Co. KG"
)


class BasicLogger(object):
    """Basic Logger Class to create a logger and add file and/or stdout handler
    depending on config with colored or json log format.

    Parameters
    ----------
    name : string
        Name of Logger to be created.
    """

    def __init__(self, name):
        self.createLogger(name)
        self.addFileHandler()
        self.addStdouthandler()

    def createLogger(self, name):
        self.log = logging.getLogger(name)
        self.initLevel()
        self.log.setLevel(getattr(logging, self.log_level))

    def addFileHandler(self):
        fileformat = self.getLogFormat("file")
        self.setLogHandler(self.log, "file", fileformat)

    def addStdouthandler(self):
        stdoutformat = self.getLogFormat("stdout")
        self.setLogHandler(self.log, "stdout", stdoutformat)

    def initLevel(self):
        level = global_config.LOG_LEVEL
        if level == 1:
            LOG_LEVEL = "ERROR"
        elif level == 2:
            LOG_LEVEL = "WARNING"
        elif level == 3:
            LOG_LEVEL = "INFO"
        else:
            LOG_LEVEL = "DEBUG"
        self.log_level = LOG_LEVEL

    def getLogFormat(self, type):
        if type == "stdout":
            format = global_config.LOG_STDOUT_FORMAT
        elif type == "file":
            format = global_config.LOG_FILE_FORMAT

        if format == "json":
            return CustomJsonFormatter(
                "%(time) %(level) %(component) %(module)%(message) %(pathname)"
                " %(lineno)%(processName) %(threadName)%(node)"
            )
        else:
            return ColoredFormatter(
                "%(log_color)s[%(asctime)s] %(levelname)-10s: "
                "%(name)s.%(module)-10s -%(message)s "
                "[in %(pathname)s:%(lineno)d]%(reset)s"
            )

    def setLogHandler(self, logger, type, format):
        if type == "stdout" and global_config.LOG_INTERFACE == "stdout":
            handler = logging.StreamHandler()
        elif type == "file":
            logfilepath = global_config.WORKER_LOGFILE.strip(
                global_config.WORKER_LOGFILE.split("/")[-1]
            )
            os.makedirs(logfilepath, exist_ok=True)
            handler = logging.handlers.RotatingFileHandler(
                global_config.WORKER_LOGFILE, maxBytes=2000000, backupCount=5
            )
        else:
            return
        handler.setFormatter(format)
        logger.addHandler(handler)


class ActiniaLogger(BasicLogger):
    """Actinia Logger which adds fluentd stream to BasicLogger if configured.

    Parameters
    ----------
    name : string
        Name of Logger to be created.
    """

    def __init__(self, name):
        BasicLogger.__init__(self, name)
        if global_config.LOG_INTERFACE == "fluentd":
            node = platform.node()
            custom_format = {
                "host": "%(hostname)s",
                "where": "%(module)s.%(funcName)s",
                "status": "%(levelname)s",
                "stack_trace": "%(exc_text)s",
            }
            fh = handler.FluentHandler(
                "%s::actinia.worker" % node,
                host=global_config.LOG_FLUENT_HOST,
                port=global_config.LOG_FLUENT_PORT,
            )
            fh_formatter = handler.FluentRecordFormatter(custom_format)
            fh.setFormatter(fh_formatter)
            self.log.addHandler(fh)


class StderrLogger(BasicLogger):
    """Standard Error Logger which collects all sys.stderr and redirects to
    logfile and console output, either in default log format (json/colored)
    or plain.

    Parameters
    ----------
    name : string
        Name of Logger to be created.
    """

    def __init__(self, name):
        self.createLogger(name)
        self.addFileHandler()

        if global_config.LOG_STDERR_FORMAT != "plain":
            self.addStdouthandler()

        sl = StreamToLogger(self.log)
        sys.stderr = sl
        # WARNING activate this only if you know what you are doing. It will
        # wrap everything that appears on stdout, even already formatted logs
        # from different loggers (e.g. from plugins) and debug print statements
        # sys.stdout = sl


class GunicornLogger(BasicLogger):
    """Collects gunicorn logs and transforms to configured format

    Parameters
    ----------
    name : string
        Name of Logger to be created.

    """

    def __init__(self, name):
        BasicLogger.__init__(self, name)
        # gunicorn already has a lot of children logger, e.g gunicorn.http,
        # gunicorn.access. These lines deactivate their default handlers.
        for name in logging.root.manager.loggerDict:
            if "gunicorn." in name:
                logging.getLogger(name).propagate = True
                logging.getLogger(name).handlers = []


class CustomJsonFormatter(jsonlogger.JsonFormatter):
    def add_fields(self, log_record, record, message_dict):
        super(CustomJsonFormatter, self).add_fields(
            log_record, record, message_dict
        )

        # (Pdb) dir(record)
        # ... 'args', 'created', 'exc_info', 'exc_text', 'filename', 'funcName'
        # ,'getMessage', 'levelname', 'levelno', 'lineno', 'message', 'module',
        # 'msecs', 'msg', 'name', 'pathname', 'process', 'processName',
        # 'relativeCreated', 'stack_info', 'thread', 'threadName']

        now = datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%fZ")
        log_record["time"] = now
        log_record["level"] = record.levelname
        log_record["component"] = record.name
        log_record["node"] = platform.node()


class StreamToLogger(object):
    """Simple logger to redirect sys.stdout and sys.stderr
    Inspired by: https://www.electricmonk.nl/log/2011/08/14/
    redirect-stdout-and-stderr-to-a-logger-in-python/
    """

    def __init__(self, logger):
        self.logger = logger

    def write(self, buffer):
        for line in buffer.rstrip().splitlines():
            # the wrapper wraps log output from different loggers. To prevent:
            if "gunicorn." in line or "actinia-gdi" in line:
                print(line.rstrip())
            else:
                # In case this is a stacktrace, it is logged line by line, each
                # wrapped by the log formatter. It is hardly readable and
                # context can get lost (but needed by e.g. kibana).
                self.logger.log(logging.CRITICAL, line.rstrip())
                if global_config.LOG_STDERR_FORMAT == "plain":
                    # This is a tradeoff for readable stacktrace in the console
                    print(line.rstrip())

    def flush(self):
        pass


def createMainLogger():
    Log = ActiniaLogger("actinia-core")
    log = Log.log
    log.info("Log level is set to '%s'", Log.log_level)

    StderrLogger("actinia-core-stderr").log
    BasicLogger("werkzeug").log
    GunicornLogger("gunicorn").log

    return log


log = createMainLogger()



# -*- coding: utf-8 -*-
#######
# actinia-core - an open source REST API for scalable, distributed, high
# performance processing of geographical data that uses GRASS GIS for
# computational tasks. For details, see https://actinia.mundialis.de/
#
# Copyright (c) 2016-2022 Sören Gebbert and mundialis GmbH & Co. KG
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
#######

"""
Asynchronous computation in specific temporary generated mapsets
with export of required map layers.
"""
import os
from actinia_core.processing.actinia_processing.ephemeral_processing import (
    EphemeralProcessing,
)
from actinia_core.core.common.process_object import Process
from actinia_core.core.common.exceptions import AsyncProcessTermination
from actinia_core.core.stac_exporter_interface import STACExporter

__license__ = "GPLv3"
__author__ = "Sören Gebbert"
__copyright__ = (
    "Copyright 2016-2022, Sören Gebbert and mundialis GmbH & Co. KG"
)
__maintainer__ = "mundialis"
__email__ = "info@mundialis.de"


class EphemeralProcessingWithExport(EphemeralProcessing):
    """
    This class processes GRASS data on the local machine in an temporary mapset
    and copies the exported results to a dedicated storage location.

    The temporary mapset will be removed by this class when the processing
    finished and the results are stored in the dedicated storage location.

    TODO: Implement the export of arbitrary files that were generated in the
          processing of the process chain
    """

    def __init__(self, rdc):
        """
        Setup the variables of this class

        Args:
            rdc (ResourceDataContainer): The data container that contains all
                                         required variables for processing

        """
        EphemeralProcessing.__init__(self, rdc)
        # Create the storage interface to store the exported resources
        self.storage_interface = rdc.create_storage_interface()

    def _export_raster(
        self,
        raster_name,
        format="COG",
        additional_options=[],
        use_raster_region=False,
    ):
        """Export a specific raster layer with r.out.gdal as GeoTiff.

        The result is stored in a temporary directory
        that is located in the temporary grass database.

        The region of the raster layer can be used for export. In this case a
        temporary region will be used for export, so that the original region
        of the mapset is not modified.
        COG-Driver: https://gdal.org/drivers/raster/cog.html

        Args:
            raster_name (str): The name of the raster layer
            format (str): COG (default; requires GDAL >= 3.1 on server), GTiff
            additional_options (list): Unused
            use_raster_region (bool): Use the region of the raster layer for
                                      export

        Returns:
            tuple: A tuple (file_name, output_path)

        Raises:
            AsyncProcessError: If a GRASS module return status is not 0

        """
        # Export the layer
        suffix = ".tif"
        # Remove a potential mapset
        file_name = raster_name.split("@")[0] + suffix

        if use_raster_region is True:
            p = Process(
                exec_type="grass",
                executable="g.region",
                executable_params=["raster=%s" % raster_name, "-g"],
                id=f"exporter_region_{raster_name}",
                stdin_source=None,
            )

            self._update_num_of_steps(1)
            self._run_module(p)

        if format == "COG":
            # check if GDAL has COG driver
            from osgeo import gdal

            driver_list = [
                gdal.GetDriver(i).ShortName
                for i in range(gdal.GetDriverCount())
            ]
            if "COG" not in driver_list:
                format = "GTiff"
                self.message_logger.info(
                    "COG driver not available, using GTiff driver"
                )

        # Save the file in the temporary directory of the temporary gisdb
        output_path = os.path.join(self.temp_file_path, file_name)

        module_name = "r.out.gdal"
        args = [
            "-fmt",
            "input=%s" % raster_name,
            "format=%s" % format,
            "output=%s" % output_path,
        ]
        create_opts = "createopt=BIGTIFF=YES,COMPRESS=LZW"

        if format == "GTiff":
            # generate overviews with compression:
            os.environ["COMPRESS_OVERVIEW"] = "LZW"
            args.append("overviews=5")
            create_opts += ",TILED=YES"

        args.append(create_opts)
        # current workaround due to color table export
        # COG bug in GDAL, see https://github.com/OSGeo/gdal/issues/2946
        # TODO: DELETE AND TEST ONCE GDAL 3.1.4 HAS BEEN RELEASED
        if format == "COG":
            args.append("-c")

        if additional_options:
            args.extend(additional_options)

        p = Process(
            exec_type="grass",
            executable=module_name,
            executable_params=args,
            id=f"exporter_raster_{raster_name}",
            stdin_source=None,
        )

        self._update_num_of_steps(1)
        self._run_module(p)

        return file_name, output_path

    def _export_strds(self, strds_name, format="GTiff"):
        """Export a specific strds layer with t.rast.export.

        The result is stored in a temporary directory
        that is located in the temporary grass database.

        Args:
            strds_name (str): The name of the strds layer
            format (str): GTiff (only option)

        Returns:
            tuple: A tuple (file_name, output_path)

        """
        suffix = ".tar.gz"
        file_name = strds_name.split("@")[0] + suffix
        output_path = os.path.join(self.temp_file_path, file_name)

        if format != "GTiff":
            format = "GTiff"
            self.message_logger.info(
                "Only GTiff driver is supported for STRDS export."
            )

        module_name = "t.rast.export"
        args = [
            "input=%s" % strds_name,
            "format=%s" % format,
            "output=%s" % output_path,
            "directory=%s" % self.temp_file_path,
            "compression=%s" % "gzip",
        ]
        # optimized for GTiff
        create_opts = "createopt=BIGTIFF=YES,COMPRESS=LZW,TILED=YES"
        args.append(create_opts)
        os.environ["COMPRESS_OVERVIEW"] = "LZW"

        p = Process(
            exec_type="grass",
            executable=module_name,
            executable_params=args,
            id=f"exporter_strds_{strds_name}",
            stdin_source=None,
        )

        self._update_num_of_steps(1)
        self._run_module(p)

        return file_name, output_path

    def _export_vector(
        self, vector_name, format="GPKG", additional_options=[]
    ):
        """
        Export a specific vector layer with v.out.ogr using a specific output
        format

        The result is stored in a temporary directory
        that is located in the temporary grass database.

        The resulting vector file will always be compressed using zip

        Args:
            vector_name (str): The name of the raster layer
            format (str): GPKG (default), GML, GeoJSON, ESRI_Shapefile, SQLite,
                          CSV
            additional_options (list): Unused

        Returns:
            tuple: A tuple (file_name, output_path)

        Raises:
            AsyncProcessError: If a GRASS module return status is not 0

        """
        # Export the layer
        prefix = ""
        if format == "GPKG":
            prefix = ".gpkg"
        if format == "GML":
            prefix = ".gml"
        if format == "GeoJSON":
            prefix = ".json"
        if format == "ESRI_Shapefile":
            prefix = ""
        if format == "SQLite":
            prefix = ".sqlite"
        if format == "CSV":
            prefix = ".csv"

        # Remove a potential mapset
        file_name = vector_name.split("@")[0] + prefix
        archive_name = file_name + ".zip"
        # switch into the temporary working directory to use relative path for
        # zip
        os.chdir(self.temp_file_path)

        module_name = "v.out.ogr"
        args = [
            "-e",
            "input=%s" % vector_name,
            "format=%s" % format,
            "output=%s" % file_name,
        ]

        if additional_options:
            args.extend(additional_options)

        # Export
        p = Process(
            exec_type="grass",
            executable=module_name,
            executable_params=args,
            id=f"exporter_vector_{vector_name}",
            stdin_source=None,
        )

        self._update_num_of_steps(1)
        self._run_module(p)

        # Compression
        compressed_output_path = os.path.join(
            self.temp_file_path, archive_name
        )

        executable = "/usr/bin/zip"
        args = ["-r", archive_name, file_name]

        p = Process(
            exec_type="exec",
            executable=executable,
            executable_params=args,
            id=f"exporter_zip_{vector_name}",
            stdin_source=None,
        )

        self._update_num_of_steps(1)
        self._run_process(p)

        return archive_name, compressed_output_path

    def _export_postgis(
        self, vector_name, dbstring, output_layer=None, additional_options=[]
    ):
        """
        Export a specific vector layer with v.out.postgis to a PostGIS database

        Args:
            vector_name (str): The name of the raster layer
            dbstring (str): The PostgreSQL database string to connect to the
                            output database
            output_layer (str): The name of the PostgreSQL database table
            additional_options (list): Unused

        Raises:
            AsyncProcessError: If a GRASS module return status is not 0

        """

        module_name = "v.out.postgis"
        args = ["-l", "input=%s" % vector_name, "output=%s" % dbstring]

        if output_layer:
            args.append("output_layer=%s" % output_layer)

        if additional_options:
            args.extend(additional_options)

        # Export
        p = Process(
            exec_type="grass",
            executable=module_name,
            executable_params=args,
            id=f"exporter_postgis_{vector_name}",
            stdin_source=None,
        )

        self._update_num_of_steps(1)
        self._run_module(p)

    def _export_file(self, tmp_file, file_name):
        """Export a specific file

        The result is stored in a temporary directory
        that is located in the temporary grass database.

        The output file will always be compressed using zip

        Args:
            tmp_file (str): The name of the temporary file generated by a
                            module
            file_name (str): The file name to be used for export

        Returns:
            tuple: A tuple (file_name, output_path)

        Raises:
            AsyncProcessError: If a GRASS module return status is not 0

        """
        # Export the file
        archive_name = file_name + ".zip"
        # switch into the temporary working directory to use relative path for
        # zip
        os.chdir(self.temp_file_path)

        # Compression
        compressed_output_path = os.path.join(
            self.temp_file_path, archive_name
        )

        executable = "/usr/bin/zip"
        args = ["-r", archive_name, tmp_file]

        p = Process(
            exec_type="exec",
            executable=executable,
            executable_params=args,
            id=f"exporter_file_{file_name}",
            stdin_source=None,
        )

        self._update_num_of_steps(1)
        self._run_process(p)

        return archive_name, compressed_output_path

    def _export_resources(self, use_raster_region=False):
        """
        Export all resources that were listed in the process chain description.

        Save all exported files in a temporary directory first, then copy the
        data to its destination after the export is finished.
        The temporary data will be finally removed.

        At the moment only raster layer export is supported.

        """

        for resource in self.resource_export_list:
            # Check for termination requests between the exports
            if (
                bool(
                    self.resource_logger.get_termination(
                        self.user_id, self.resource_id, self.iteration
                    )
                )
                is True
            ):
                raise AsyncProcessTermination(
                    "Resource export was terminated by user request"
                )

            # Raster export
            if resource["export"]["type"] in [
                "raster",
                "vector",
                "file",
                "strds",
            ]:
                output_type = resource["export"]["type"]
                output_path = None

                # Legacy code
                if "name" in resource:
                    file_name = resource["name"]
                if "value" in resource:
                    file_name = resource["value"]

                if output_type == "raster":
                    message = "Export raster layer <%s> with format %s" % (
                        file_name,
                        resource["export"]["format"],
                    )
                    self._send_resource_update(message)
                    output_name, output_path = self._export_raster(
                        raster_name=file_name,
                        format=resource["export"]["format"],
                        use_raster_region=use_raster_region,
                    )

                elif output_type == "vector":
                    if "PostgreSQL" in resource["export"]["format"]:
                        dbstring = resource["export"]["dbstring"]
                        output_layer = None
                        if "output_layer" in resource["export"]:
                            output_layer = resource["export"]["output_layer"]

                        message = (
                            "Export vector layer <%s> to PostgreSQL database"
                            % (file_name)
                        )
                        self._send_resource_update(message)
                        self._export_postgis(
                            vector_name=file_name,
                            dbstring=dbstring,
                            output_layer=output_layer,
                        )
                        # continue
                    else:
                        message = "Export vector layer <%s> with format %s" % (
                            file_name,
                            resource["export"]["format"],
                        )
                        self._send_resource_update(message)
                        output_name, output_path = self._export_vector(
                            vector_name=file_name,
                            format=resource["export"]["format"],
                        )
                elif output_type == "file":
                    file_name = resource["file_name"]
                    tmp_file = resource["tmp_file"]
                    output_name, output_path = self._export_file(
                        tmp_file=tmp_file, file_name=file_name
                    )
                elif output_type == "strds":
                    message = "Export strds layer <%s> with format %s" % (
                        file_name,
                        resource["export"]["format"],
                    )
                    self._send_resource_update(message)
                    output_name, output_path = self._export_strds(
                        strds_name=file_name,
                        format=resource["export"]["format"],
                    )
                else:
                    raise AsyncProcessTermination(
                        "Unknown export format %s" % output_type
                    )

                message = "Moving generated resources to final destination"
                self._send_resource_update(message)

                # Store the temporary file in the resource storage
                # and receive the resource URL
                if output_path is not None:
                    resource_url = self.storage_interface.store_resource(
                        output_path
                    )
                    self.resource_url_list.append(resource_url)

                    if "metadata" in resource:
                        if resource["metadata"]["format"] == "STAC":
                            stac = STACExporter()

                            stac_catalog = stac.stac_builder(
                                resource_url, file_name, output_type
                            )
                            self.resource_url_list.append(stac_catalog)

    def _execute(self, skip_permission_check=False):
        """Overwrite this function in subclasses

        Overwrite this function in subclasses

            - Setup user credentials
            - Setup the storage interface
            - Analyse the process chain
            - Initialize and create the temporal database and mapset
            - Run the modules
            - Export the results
            - Cleanup

        """
        # Setup the user credentials and logger
        self._setup()

        # Create and check the resource directory
        self.storage_interface.setup()

        EphemeralProcessing._execute(self)

        # Export all resources and generate the finish response
        self._export_resources()

    def _final_cleanup(self):
        """
        Overwrite this function in subclasses to perform the final cleanup
        """
        # Clean up and remove the temporary gisdbase
        self._cleanup()
        # Remove resource directories
        if "error" in self.run_state or "terminated" in self.run_state:
            self.storage_interface.remove_resources()
